import cv2
import time

from numpy.typing import NDArray
from typing import Optional, Tuple

from PySide6.QtCore import Qt, Signal, QThread
from PySide6.QtGui import QImage
from pykinect_recorder.main._pyk4a.k4arecord.playback import Playback


class PlaybackSensors(QThread):
    rgb_updated_frame = Signal(QImage)
    depth_updated_frame = Signal(QImage)
    ir_updated_frame = Signal(QImage)
    Time = Signal(float)
    AccData = Signal(list)
    gyro_data = Signal(list)
    Fps = Signal(float)
    
    def __init__(self, playback: Playback, parent=None) -> None:
        QThread.__init__(self, parent)
        self.playback = playback
        self.is_run = None

    def run(self):      
        while self.is_run:
            start_t = time.time()
            _, current_frame = self.playback.update()

            # (Success flag, numpy data)
            current_rgb_frame = current_frame.get_color_image()
            current_depth_frame = current_frame.get_depth_image()
            current_ir_frame = current_frame.get_ir_image()
            # current_imu_data = self.playback.update_imu()

            if current_rgb_frame[0]:
                rgb_frame = current_rgb_frame[1]
                rgb_frame = cv2.cvtColor(rgb_frame, cv2.COLOR_BGR2RGB)
                
                h, w, ch = rgb_frame.shape
                rgb_frame = QImage(rgb_frame, w, h, ch * w, QImage.Format_RGB888)
                scaled_rgb_frame = rgb_frame.scaled(720, 440, Qt.KeepAspectRatio)
                self.rgb_updated_frame.emit(scaled_rgb_frame)

            if current_depth_frame[0]:
                depth_frame = self._colorize(
                    current_depth_frame[1], (None, 5000), cv2.COLORMAP_HSV
                )
                h, w, ch = depth_frame.shape

                depth_frame = QImage(depth_frame, w, h, w * ch, QImage.Format_RGB888)
                scaled_depth_frame = depth_frame.scaled(440, 440, Qt.KeepAspectRatio)
                self.depth_updated_frame.emit(scaled_depth_frame)

            if current_ir_frame[0]:
                ir_frame = self._colorize(
                    current_ir_frame[1], (None, 5000), cv2.COLORMAP_BONE
                )
                h, w, ch = ir_frame.shape

                ir_frame = QImage(ir_frame, w, h, w * ch, QImage.Format_RGB888)
                scaled_ir_frame = ir_frame.scaled(440, 440, Qt.KeepAspectRatio)
                self.ir_updated_frame.emit(scaled_ir_frame)

            end_time = time.time()
            # acc_time = current_imu_data.acc_time
            # acc_data = current_imu_data.acc
            # gyro_data = current_imu_data.gyro
            fps = 1/(end_time-start_t)

            self.Fps.emit(fps)
            # self.Time.emit(acc_time/1e6)
            # self.AccData.emit(acc_data)
            # self.gyro_data.emit(gyro_data)

        if cv2.waitKey(30) == ord("q"):
            self.is_run = False
            self.quit()

    def _colorize(
        self,
        image: NDArray,
        clipping_range: Tuple[Optional[int], Optional[int]] = (None, None),
        colormap: int = cv2.COLORMAP_HSV,
    ) -> NDArray:
        if clipping_range[0] or clipping_range[1]:
            img = image.clip(clipping_range[0], clipping_range[1])  # type: ignore
        else:
            img = image.copy()
        img = cv2.normalize(img, None, 0, 255, cv2.NORM_MINMAX, dtype=cv2.CV_8U)
        img = cv2.applyColorMap(img, colormap)
        return img


import numpy as np
from pykinect_azure.k4abt._k4abtTypes import K4ABT_JOINT_NAMES


class Joint2d:
    def __init__(self, joint2d_handle, id):
        if joint2d_handle:
            self._handle = joint2d_handle
            self.position = joint2d_handle.position.xy
            self.confidence_level = joint2d_handle.confidence_level
            self.id = id
            self.name = self.get_name()

    def __del__(self):
        self.destroy()

    def numpy(self):
        return np.array([self.position.x, self.position.y])

    def is_valid(self):
        return self._handle

    def handle(self):
        return self._handle

    def destroy(self):
        if self.is_valid():
            self._handle = None

    def get_coordinates(self):
        return (int(self.position.x), int(self.position.y))

    def get_name(self):
        return K4ABT_JOINT_NAMES[self.id]

    def __str__(self):
        """Print the current settings and a short explanation"""
        message = (
            f"{self.name} Joint 2d info: \n"
            f"\tPixel: [{self.position.x},{self.position.y}]\n"
            f"\tconfidence: {self.confidence_level} \n\n"
        )
        return message



from ..utils import getdict


class ImuSample:
    def __init__(self, imu_sample_struct):
        self._struct = imu_sample_struct
        self.parse_data()

    def __del__(self):
        self.reset()

    def is_valid(self):
        return self._struct

    def struct(self):
        return self._struct

    def reset(self):
        if self.is_valid():
            self._struct = None

    def parse_data(self):
        self.imu_sample_dict = getdict(self._struct)

        # Convert the acc and gyro dicts to numpy array
        self.imu_sample_dict["acc_sample"] = self.imu_sample_dict["acc_sample"]["v"]
        self.imu_sample_dict["gyro_sample"] = self.imu_sample_dict["gyro_sample"]["v"]

    @property
    def temp(self):
        return self.get_temp()

    @property
    def acc(self):
        return self.get_acc()

    @property
    def acc_time(self):
        return self.get_acc_time()

    @property
    def gyro(self):
        return self.get_gyro()

    @property
    def gyro_time(self):
        return self.get_gyro_time()

    def get_temp(self):
        return self.imu_sample_dict["temperature"]

    def get_acc(self):
        return self.imu_sample_dict["acc_sample"]

    def get_acc_time(self):
        return self.imu_sample_dict["acc_timestamp_usec"]

    def get_gyro(self):
        return self.imu_sample_dict["gyro_sample"]

    def get_gyro_time(self):
        return self.imu_sample_dict["gyro_timestamp_usec"]

    def get_sample(self):
        return self.imu_sample_dict



import datetime
import os
import sys
from pathlib import Path
from typing import Optional, Tuple

import cv2

import numpy as np
from numpy.typing import NDArray
from pyk4a import PyK4A, PyK4ARecord
from PySide6.QtCore import Qt, QThread, Signal
from PySide6.QtGui import QImage


class RecordVideo(QThread):
    rgb_updated_frame = Signal(QImage)
    depth_updated_frame = Signal(QImage)

    def __init__(self, config, parent=None):
        QThread.__init__(self, parent)
        self.config = config
        self.status = True
        self.set_filename()

    def set_filename(self) -> None:
        base_path = os.path.join(Path.home(), "Videos")

        filename = datetime.datetime.now()
        filename = filename.strftime("%Y_%m_%d_%H_%M_%S")

        self.filename_video = os.path.join(base_path, f"{filename}.mkv")
        self.filename_audio = os.path.join(base_path, f"{filename}.wav")
        if sys.flags.debug:
            print(base_path, self.filename_video, self.filename_audio)

    def colorize(
        self,
        image: NDArray,
        clipping_range: Tuple[Optional[int], Optional[int]] = (None, None),
        colormap: int = cv2.COLORMAP_HSV,
    ) -> NDArray:
        if clipping_range[0] or clipping_range[1]:
            img = image.clip(clipping_range[0], clipping_range[1])  # type: ignore
        else:
            img = image.copy()
        img = cv2.normalize(img, None, 0, 255, cv2.NORM_MINMAX, dtype=cv2.CV_8U)
        img = cv2.applyColorMap(img, colormap)
        return img

    def run(self):
        azure_device = PyK4A(config=self.config, device_id=0)
        azure_device.start()
        record = PyK4ARecord(
            device=azure_device, config=self.config, path=self.filename_video
        )
        record.create()
        while self.status:
            current_frame = azure_device.get_capture()
            record.write_capture(current_frame)

            if np.any(current_frame.color):
                color_frame = current_frame.color[:, :, :3].copy()
                color_frame = cv2.cvtColor(color_frame, cv2.COLOR_BGR2RGB)

                h, w, ch = color_frame.shape
                if sys.flags.debug:
                    print(h, w)
                img = QImage(color_frame, w, h, ch * w, QImage.Format_RGB888)
                scaled_img = img.scaled(640, 480, Qt.KeepAspectRatio)
                self.rgb_updated_frame.emit(scaled_img)

            if np.any(current_frame.depth):
                depth_frame = self.colorize(
                    current_frame.depth, (None, 5000), cv2.COLORMAP_HSV
                )
                h, w, ch = depth_frame.shape
                if sys.flags.debug:
                    print(h, w)

                depth_frame = QImage(depth_frame, w, h, w * ch, QImage.Format_RGB888)
                scaled_depth_frame = depth_frame.scaled(400, 360, Qt.KeepAspectRatio)
                self.depth_updated_frame.emit(scaled_depth_frame)



from PySide6.QtWidgets import (
    QFrame
)

class ViewerSolution(QFrame):
    def __init__(self):
        super().__init__()
        self.setFixedSize(1200, 1000)
        self.setStyleSheet("background-color: black;")
        pass



import ctypes


# K4A_DECLARE_HANDLE(k4a_record_t);
class _handle_k4a_record_t(ctypes.Structure):
    _fields_ = [
        ("_rsvd", ctypes.c_size_t),
    ]

# streaming frame record 
k4a_record_t = ctypes.POINTER(_handle_k4a_record_t)


# K4A_DECLARE_HANDLE(k4a_playback_t);
class _handle_k4a_playback_t(ctypes.Structure):
    _fields_ = [
        ("_rsvd", ctypes.c_size_t),
    ]

# watching video which recorded by k4a_record
k4a_playback_t = ctypes.POINTER(_handle_k4a_playback_t)


# K4A_DECLARE_HANDLE(k4a_playback_data_block_t);
class _handle_k4a_playback_data_block_t(ctypes.Structure):
    _fields_ = [
        ("_rsvd", ctypes.c_size_t),
    ]

# Handle to a block of data read from k4a_playback
k4a_playback_data_block_t = ctypes.POINTER(_handle_k4a_playback_data_block_t)


# class k4a_stream_result_t(CtypeIntEnum):
k4a_stream_result_t = ctypes.c_int
K4A_STREAM_RESULT_SUCCEEDED = 0
K4A_STREAM_RESULT_FAILED = 1
K4A_STREAM_RESULT_EOF = 2

# handling record video
k4a_playback_seek_origin_t = ctypes.c_int
K4A_PLAYBACK_SEEK_BEGIN = 0
K4A_PLAYBACK_SEEK_END = 1
K4A_PLAYBACK_SEEK_DEVICE_TIME = 2


class _k4a_record_configuration_t(ctypes.Structure):
    _fields_ = [
        ("color_format", ctypes.c_int),
        ("color_resolution", ctypes.c_int),
        ("depth_mode", ctypes.c_int),
        ("camera_fps", ctypes.c_int),
        ("color_track_enabled", ctypes.c_bool),
        ("depth_track_enabled", ctypes.c_bool),
        ("ir_track_enabled", ctypes.c_bool),
        ("imu_track_enabled", ctypes.c_bool),
        ("depth_delay_off_color_usec", ctypes.c_int32),
        ("wired_sync_mode", ctypes.c_int),
        ("subordinate_delay_off_master_usec", ctypes.c_uint32),
        ("start_timestamp_offset_usec", ctypes.c_uint32),
    ]


k4a_record_configuration_t = _k4a_record_configuration_t


class _k4a_record_video_settings_t(ctypes.Structure):
    _fields_ = [
        ("width", ctypes.c_ulong),
        ("height", ctypes.c_ulong),
        ("frame_rate", ctypes.c_ulong),
    ]


k4a_record_video_settings_t = _k4a_record_video_settings_t


class _k4a_record_subtitle_settings_t(ctypes.Structure):
    _fields_ = [
        ("high_freq_data", ctypes.c_bool),
    ]


k4a_record_subtitle_settings_t = _k4a_record_subtitle_settings_t



from ..k4a import _k4a


class RecordConfiguration:
    def __init__(self, configuration_handle=None):
        self._handle = configuration_handle

    def handle(self):
        return self._handle

    def __getattr__(self, name):
        """Pass the handle parameter, when asked"""

        if name == "_handle":
            return self.__dict__[name]
        else:
            return self._handle.__dict__[name]

    def __str__(self):
        """Print the current settings and a short explanation"""
        message = (
            "Record configuration: \n"
            f"\tcolor_format: {self._handle.color_format} \n\t(0:JPG, 1:NV12, 2:YUY2, 3:BGRA32)\n\n"
            f"\tcolor_resolution: {self._handle.color_resolution} \n\t(0:OFF, 1:720p, 2:1080p, 3:1440p, 4:1536p, 5:2160p, 6:3072p)\n\n"
            f"\tdepth_mode: {self._handle.depth_mode} \n\t(0:OFF, 1:NFOV_2X2BINNED, 2:NFOV_UNBINNED,3:WFOV_2X2BINNED, 4:WFOV_UNBINNED, 5:Passive IR)\n\n"
            f"\tcamera_fps: {self._handle.camera_fps} \n\t(0:5 FPS, 1:15 FPS, 2:30 FPS)\n\n"
            f"\tcolor_track_enabled: {self._handle.color_track_enabled} \n\t(True of False). If Color camera images exist\n\n"
            f"\tdepth_track_enabled: {self._handle.depth_track_enabled} \n\t(True of False). If Depth camera images exist\n\n"
            f"\tir_track_enabled: {self._handle.ir_track_enabled} \n\t(True of False). If IR camera images exist\n\n"
            f"\timu_track_enabled: {self._handle.imu_track_enabled} \n\t(True of False). If IMU samples exist\n\n"
            f"\tdepth_delay_off_color_usec: {self._handle.depth_delay_off_color_usec} us. \n\tDelay between the color image and the depth image\n\n"
            f"\twired_sync_mode: {self._handle.wired_sync_mode}\n\t(0:Standalone mode, 1:Master mode, 2:Subordinate mode)\n\n"
            f"\tsubordinate_delay_off_master_usec: {self._handle.subordinate_delay_off_master_usec} us.\n\tThe external synchronization timing.\n\n"
            f"\tstart_timestamp_offset_usec: {self._handle.start_timestamp_offset_usec} us. \n\tStart timestamp offset.\n\n"
        )
        return message



import click
from pykinect_recorder.renderer import MainWindow
import qdarktheme
from PySide6.QtWidgets import QApplication
 

CONTEXT_SETTINGS = dict(help_option_names=['-h','--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    pass


# @click.command(help="pykinect")
# @click.option("--open", is_flag=True)
# def check(open):
#     if open:
#         print('sibal')
#         # run()


def main():
    app = QApplication()
    qdarktheme.setup_theme()
    main_window = MainWindow()
    main_window.show()
    app.exec()


if __name__ == '__main__':
    main()


import numpy as np

from pykinect_azure.k4abt._k4abtTypes import K4ABT_JOINT_COUNT
from pykinect_azure.k4abt.joint import Joint


class Body:
    def __init__(self, skeleton_handle):
        if skeleton_handle:
            self._handle = skeleton_handle
            self.initialize()

    def json(self):
        return self._handle.__iter__()

    def numpy(self):
        return np.array([joint.numpy() for joint in self.joints])

    def __del__(self):
        self.destroy()

    def is_valid(self):
        return self._handle

    def handle(self):
        return self._handle

    def destroy(self):
        if self.is_valid():
            self._handle = None

    def initialize(self):
        joints = np.ndarray((K4ABT_JOINT_COUNT,), dtype=np.object_)

        for i in range(K4ABT_JOINT_COUNT):
            joints[i] = Joint(self._handle.skeleton.joints[i], i)

        self.joints = joints

    def __str__(self):
        """Print the current settings and a short explanation"""
        message = "\nBody:\n" + "".join(f"{joint.__str__()}" for joint in self.joints)

        return message



import cv2
import sys
import time
import queue
import sounddevice as sd
import soundfile as sf

from numpy.typing import NDArray
from typing import Optional, Tuple
from PySide6.QtCore import Qt, Signal, QThread
from PySide6.QtGui import QImage
from pykinect_recorder.main._pyk4a.k4a import Device
from PySide6.QtMultimedia import (
    QAudioFormat, QAudioSource, QMediaDevices,
)


RESOLUTION = 4
q = queue.Queue()


def callback(indata, frames, time, status):
    global q
    """This is called (from a separate thread) for each audio block."""
    if status:
        print(status, file=sys.stderr)
    q.put(indata.copy())


class Pyk4aThread(QThread):
    global queue
    rgb_updated_frame = Signal(QImage)
    depth_updated_frame = Signal(QImage)
    ir_updated_frame = Signal(QImage)
    Time = Signal(float)
    AccData = Signal(list)
    gyro_data = Signal(list)
    Fps = Signal(float)
    Audio = Signal(list)
    
    def __init__(self, device: Device, audio_record = None, parent=None) -> None:
        QThread.__init__(self, parent)
        self.device = device
        self.is_run = None
        
        self.input_devices = QMediaDevices.audioInputs()
        self.audio_input = None
        self.audio_file = None
        self.audio_record = audio_record
    
    def run(self):
        self.readyAudio()
        self.io_device = self.audio_input.start()

        with sf.SoundFile(
            self.audio_file,
            mode="x",
            samplerate=44100,
            channels=2,
        ) as file:
            with sd.InputStream(
                samplerate=44100,
                device=0,
                channels=2,
                callback=callback,
            ):
                while self.is_run:
                    start_t = time.time()
                    current_frame = self.device.update()
                    file.write(q.get())

                    # (Success flag, numpy data)
                    current_rgb_frame = current_frame.get_color_image()
                    current_depth_frame = current_frame.get_depth_image()
                    current_ir_frame = current_frame.get_ir_image()
                    current_imu_data = self.device.update_imu()

                    if current_rgb_frame[0]:
                        rgb_frame = current_rgb_frame[1]
                        rgb_frame = cv2.cvtColor(rgb_frame, cv2.COLOR_BGR2RGB)
                        
                        h, w, ch = rgb_frame.shape
                        rgb_frame = QImage(rgb_frame, w, h, ch * w, QImage.Format_RGB888)
                        scaled_rgb_frame = rgb_frame.scaled(720, 440, Qt.KeepAspectRatio)
                        self.rgb_updated_frame.emit(scaled_rgb_frame)

                    if current_depth_frame[0]:
                        depth_frame = self._colorize(
                            current_depth_frame[1], (None, 5000), cv2.COLORMAP_HSV
                        )
                        h, w, ch = depth_frame.shape

                        depth_frame = QImage(depth_frame, w, h, w * ch, QImage.Format_RGB888)
                        scaled_depth_frame = depth_frame.scaled(440, 440, Qt.KeepAspectRatio)
                        self.depth_updated_frame.emit(scaled_depth_frame)

                    if current_ir_frame[0]:
                        ir_frame = self._colorize(
                            current_ir_frame[1], (None, 5000), cv2.COLORMAP_BONE
                        )
                        h, w, ch = ir_frame.shape

                        ir_frame = QImage(ir_frame, w, h, w * ch, QImage.Format_RGB888)
                        scaled_ir_frame = ir_frame.scaled(440, 440, Qt.KeepAspectRatio)
                        self.ir_updated_frame.emit(scaled_ir_frame)

                    end_time = time.time()
                    acc_time = current_imu_data.acc_time
                    acc_data = current_imu_data.acc
                    gyro_data = current_imu_data.gyro
                    fps = 1/(end_time-start_t)

                    self.Fps.emit(fps)
                    self.Time.emit(acc_time/1e6)
                    self.AccData.emit(acc_data)
                    self.gyro_data.emit(gyro_data)

                    # audio
                    data = self.io_device.readAll()
                    available_samples = data.size() // RESOLUTION
                    self.Audio.emit([data, available_samples])

        if self.audio_record is None:
            import os
            os.remove(self.audio_file)

        self.audio_input.stop()
        self.io_device = None

    def _colorize(
        self,
        image: NDArray,
        clipping_range: Tuple[Optional[int], Optional[int]] = (None, None),
        colormap: int = cv2.COLORMAP_HSV,
    ) -> NDArray:
        if clipping_range[0] or clipping_range[1]:
            img = image.clip(clipping_range[0], clipping_range[1])  # type: ignore
        else:
            img = image.copy()
        img = cv2.normalize(img, None, 0, 255, cv2.NORM_MINMAX, dtype=cv2.CV_8U)
        img = cv2.applyColorMap(img, colormap)
        return img
    
    def readyAudio(self) -> None:
        format_audio = QAudioFormat()
        format_audio.setSampleRate(44200)
        format_audio.setChannelCount(3)
        format_audio.setSampleFormat(QAudioFormat.SampleFormat.UInt8)

        self.audio_input = QAudioSource(self.input_devices[0], format_audio, self)


import os
from glob import glob
from pathlib import Path

from PySide6.QtCore import Qt, Signal, Slot
from PySide6.QtWidgets import (
    QFrame, QWidget, QVBoxLayout, QScrollArea, 
    QPushButton, QHBoxLayout, QFileDialog
)
from ..common_widgets import Label, PushButton
from ..signals import all_signals


class ExplorerSidebar(QFrame):
    def __init__(self) -> None:
        super().__init__()
        self.setStyleSheet("background-color: #252526;")
        
        self.layout_main = QVBoxLayout()
        self.layout_title = QHBoxLayout()
        self.base_path = os.path.join(Path.home(), "Videos/")
        # title
        self.label_dirpath = Label("")
        self.label_dirpath.setFixedSize(200, 50)
        self.btn_search = PushButton("경로\n탐색")
        self.btn_search.setFixedSize(50, 50)
        self.layout_title.addWidget(self.label_dirpath)
        self.layout_title.addWidget(self.btn_search)
        self.btn_search.clicked.connect(self.search_file)
        
        # scroll area
        self.layout_scroll = QScrollArea()
        self.layout_scroll.setWidgetResizable(True)

        self.layout_main.addLayout(self.layout_title)
        self.layout_main.addWidget(self.layout_scroll)
        self.setMaximumHeight(1080)
        self.setFixedWidth(300)
        self.setLayout(self.layout_main)
        self.set_scrollarea(self.base_path)

    def search_file(self) -> None:
        _dirNames = QFileDialog.getExistingDirectory(
            self, 
            "Open Data Files", 
            ".",
            QFileDialog.ShowDirsOnly
        )
        print(_dirNames)
        self.base_path = _dirNames
        self.label_dirpath.setText(_dirNames)
        self.set_scrollarea(_dirNames)

    def set_scrollarea(self, filedirs: str) -> None:
        self._widget = QWidget()
        layout_file = QVBoxLayout()
        layout_file.setAlignment(Qt.AlignmentFlag.AlignTop)

        for filedir in Path(filedirs).iterdir():
            _filename = str(filedir).split("\\")[-1]
            if _filename[-4:] == '.mkv':
                fileinfo = _FileInfo()
                fileinfo.label_name.setText(_filename)
                fsize = os.path.getsize(filedir) / (2**30)  # byte -> GB
                fileinfo.label_stor.setText("파일 크기 : %.2fGB" %(fsize))
                layout_file.addWidget(fileinfo)
                fileinfo.Filename.connect(self.emit_file_path)

        self._widget.setLayout(layout_file)
        self.layout_scroll.setWidget(self._widget)

    @Slot(str)
    def emit_file_path(self, filename) -> None:
        tmp = os.path.join(self.base_path, filename)
        all_signals.playback_filepath.emit(tmp)
        

class _FileInfo(QPushButton):
    Filename = Signal(str)
    def __init__(self) -> None:
        super().__init__()
        self.setFixedSize(240,100)
        self.setObjectName("FileInfo")
        self.setStyleSheet("""
            QPushButton#FileInfo {
                border-color: white;
            }
        """)

        layout_main = QVBoxLayout()
        self.label_name = Label("파일 이름", "Arial", 10, Qt.AlignmentFlag.AlignCenter)
        self.label_stor = Label("총 용량", "Arial", 10, Qt.AlignmentFlag.AlignCenter)
        self.label_name.setWordWrap(True)

        layout_main.addWidget(self.label_name)
        layout_main.addWidget(self.label_stor)
        self.setLayout(layout_main)
        self.clicked.connect(self.emitName)
    
    def emitName(self) -> None:
        self.Filename.emit(self.label_name.text())
    


import ctypes
import platform
import sys
import traceback

from pykinect_azure.k4abt._k4abtTypes import *
from pykinect_azure.k4a._k4atypes import k4a_calibration_t, k4a_capture_t, k4a_image_t

k4abt_dll = None


def setup_library(module_k4abt_path):
    global k4abt_dll

    try:
        k4abt_dll = ctypes.CDLL(module_k4abt_path)

    except Exception as e:
        print("Failed to load body tracker library", e)
        sys.exit(1)
    setup_onnx_provider()


def setup_onnx_provider():
    if platform.system() == "Windows":
        setup_onnx_provider_windows()
    elif platform.system() == "Linux":
        setup_onnx_provider_linux()


def setup_onnx_provider_linux():
    k4abt_tracker_default_configuration.processing_mode = (
        K4ABT_TRACKER_PROCESSING_MODE_GPU_CUDA
    )
    try:
        ctypes.cdll.LoadLibrary("libonnxruntime_providers_cuda.so")
    except Exception as e:
        ctypes.cdll.LoadLibrary("libonnxruntime.so.1.10.0")


def setup_onnx_provider_windows():
    try:
        ctypes.cdll.LoadLibrary(
            "C:/Program Files/Azure Kinect Body Tracking SDK/tools/directml.dll"
        )
    except Exception as e:
        try:
            ctypes.cdll.LoadLibrary(
                "C:/Program Files/Azure Kinect Body Tracking SDK/sdk/windows-desktop/amd64/release/bin/onnxruntime_providers_cuda.dll"
            )
            k4abt_tracker_default_configuration.processing_mode = (
                K4ABT_TRACKER_PROCESSING_MODE_GPU_CUDA
            )
        except Exception as e:
            k4abt_tracker_default_configuration.processing_mode = (
                K4ABT_TRACKER_PROCESSING_MODE_CPU
            )


def k4abt_tracker_create(sensor_calibration, config, tracker_handle):
    """
    K4ABT_EXPORT k4a_result_t k4abt_tracker_create(const k4a_calibration_t* sensor_calibration,
                                                                                    k4abt_tracker_configuration_t config,
                                                                                    k4abt_tracker_t* tracker_handle);
    """
    _k4abt_tracker_create = k4abt_dll.k4abt_tracker_create
    _k4abt_tracker_create.restype = ctypes.c_int
    _k4abt_tracker_create.argtypes = (
        ctypes.POINTER(k4a_calibration_t),
        k4abt_tracker_configuration_t,
        ctypes.POINTER(k4abt_tracker_t),
    )

    return _k4abt_tracker_create(sensor_calibration, config, tracker_handle)


def k4abt_tracker_destroy(tracker_handle):
    # K4ABT_EXPORT void k4abt_tracker_destroy(k4abt_tracker_t tracker_handle);

    _k4abt_tracker_destroy = k4abt_dll.k4abt_tracker_destroy
    _k4abt_tracker_destroy.argtypes = (k4abt_tracker_t,)

    _k4abt_tracker_destroy(tracker_handle)


def k4abt_tracker_set_temporal_smoothing(tracker_handle, smoothing_factor):
    # K4ABT_EXPORT void k4abt_tracker_set_temporal_smoothing(k4abt_tracker_t tracker_handle, float smoothing_factor);

    _k4abt_tracker_set_temporal_smoothing = (
        k4abt_dll.k4abt_tracker_set_temporal_smoothing
    )
    _k4abt_tracker_set_temporal_smoothing.argtypes = (k4abt_tracker_t, ctypes.c_float)

    _k4abt_tracker_set_temporal_smoothing(tracker_handle, smoothing_factor)


def k4abt_tracker_enqueue_capture(tracker_handle, sensor_capture_handle, timeout_in_ms):
    """
    K4ABT_EXPORT k4a_wait_result_t k4abt_tracker_enqueue_capture(k4abt_tracker_t tracker_handle,
                                                                                                                    k4a_capture_t sensor_capture_handle,
                                                                                                                    int32_t timeout_in_ms);
    """

    _k4abt_tracker_enqueue_capture = k4abt_dll.k4abt_tracker_enqueue_capture
    _k4abt_tracker_enqueue_capture.restype = ctypes.c_int
    _k4abt_tracker_enqueue_capture.argtypes = (
        k4abt_tracker_t,
        k4a_capture_t,
        ctypes.c_int32,
    )

    return _k4abt_tracker_enqueue_capture(
        tracker_handle, sensor_capture_handle, timeout_in_ms
    )


def k4abt_tracker_pop_result(tracker_handle, body_frame_handle, timeout_in_ms):
    """
    K4ABT_EXPORT k4a_wait_result_t k4abt_tracker_pop_result(k4abt_tracker_t tracker_handle,
                                                                                                                            k4abt_frame_t* body_frame_handle,
                                                                                                                            int32_t timeout_in_ms);

    """

    _k4abt_tracker_pop_result = k4abt_dll.k4abt_tracker_pop_result
    _k4abt_tracker_pop_result.restype = ctypes.c_int
    _k4abt_tracker_pop_result.argtypes = (
        k4abt_tracker_t,
        ctypes.POINTER(k4abt_frame_t),
        ctypes.c_int32,
    )

    return _k4abt_tracker_pop_result(tracker_handle, body_frame_handle, timeout_in_ms)


def k4abt_tracker_shutdown(tracker_handle):
    # K4ABT_EXPORT void k4abt_tracker_shutdown(k4abt_tracker_t tracker_handle);

    _k4abt_tracker_shutdown = k4abt_dll.k4abt_tracker_shutdown
    _k4abt_tracker_shutdown.argtypes = (k4abt_tracker_t,)

    _k4abt_tracker_shutdown(tracker_handle)


def k4abt_frame_release(body_frame_handle):
    # K4ABT_EXPORT void k4abt_frame_release(k4abt_frame_t body_frame_handle);

    _k4abt_frame_release = k4abt_dll.k4abt_frame_release
    _k4abt_frame_release.argtypes = (k4abt_frame_t,)

    _k4abt_frame_release(body_frame_handle)


def k4abt_frame_reference(body_frame_handle):
    # K4ABT_EXPORT void k4abt_frame_reference(k4abt_frame_t body_frame_handle);

    _k4abt_frame_reference = k4abt_dll.k4abt_frame_reference
    _k4abt_frame_reference.argtypes = (k4abt_frame_t,)

    _k4abt_frame_reference(body_frame_handle)


def k4abt_frame_get_num_bodies(body_frame_handle):
    # K4ABT_EXPORT uint32_t k4abt_frame_get_num_bodies(k4abt_frame_t body_frame_handle);

    _k4abt_frame_get_num_bodies = k4abt_dll.k4abt_frame_get_num_bodies
    _k4abt_frame_get_num_bodies.restype = ctypes.c_uint32
    _k4abt_frame_get_num_bodies.argtypes = (k4abt_frame_t,)

    return _k4abt_frame_get_num_bodies(body_frame_handle)


def k4abt_frame_get_body_skeleton(body_frame_handle, index, skeleton):
    # K4ABT_EXPORT k4a_result_t k4abt_frame_get_body_skeleton(k4abt_frame_t body_frame_handle, uint32_t index, k4abt_skeleton_t* skeleton);

    _k4abt_frame_get_body_skeleton = k4abt_dll.k4abt_frame_get_body_skeleton
    _k4abt_frame_get_body_skeleton.restype = ctypes.c_int
    _k4abt_frame_get_body_skeleton.argtypes = (
        k4abt_frame_t,
        ctypes.c_uint32,
        ctypes.POINTER(k4abt_skeleton_t),
    )

    return _k4abt_frame_get_body_skeleton(body_frame_handle, index, skeleton)


def k4abt_frame_get_body_id(body_frame_handle, index):
    # K4ABT_EXPORT uint32_t k4abt_frame_get_body_id(k4abt_frame_t body_frame_handle, uint32_t index);

    _k4abt_frame_get_body_id = k4abt_dll.k4abt_frame_get_body_id
    _k4abt_frame_get_body_id.restype = ctypes.c_uint32
    _k4abt_frame_get_body_id.argtypes = (k4abt_frame_t, ctypes.c_uint32)

    return _k4abt_frame_get_body_id(body_frame_handle, index)


def k4abt_frame_get_device_timestamp_usec(body_frame_handle):
    # K4ABT_EXPORT uint64_t k4abt_frame_get_device_timestamp_usec(k4abt_frame_t body_frame_handle);

    _k4abt_frame_get_device_timestamp_usec = (
        k4abt_dll.k4abt_frame_get_device_timestamp_usec
    )
    _k4abt_frame_get_device_timestamp_usec.restype = ctypes.c_uint64
    _k4abt_frame_get_device_timestamp_usec.argtypes = (k4abt_frame_t,)

    return _k4abt_frame_get_device_timestamp_usec(body_frame_handle)


def k4abt_frame_get_body_index_map(body_frame_handle):
    #  K4ABT_EXPORT k4a_image_t k4abt_frame_get_body_index_map(k4abt_frame_t body_frame_handle);

    _k4abt_frame_get_body_index_map = k4abt_dll.k4abt_frame_get_body_index_map
    _k4abt_frame_get_body_index_map.restype = k4a_image_t
    _k4abt_frame_get_body_index_map.argtypes = (k4abt_frame_t,)

    return _k4abt_frame_get_body_index_map(body_frame_handle)


def k4abt_frame_get_capture(body_frame_handle):
    # K4ABT_EXPORT k4a_capture_t k4abt_frame_get_capture(k4abt_frame_t body_frame_handle);

    _k4abt_frame_get_capture = k4abt_dll.k4abt_frame_get_capture
    _k4abt_frame_get_capture.restype = k4a_capture_t
    _k4abt_frame_get_capture.argtypes = (k4abt_frame_t,)

    return _k4abt_frame_get_capture(body_frame_handle)


def VERIFY(result, error):
    if result != K4ABT_RESULT_SUCCEEDED:
        print(error)
        traceback.print_stack()
        sys.exit(1)



from ..k4arecord import _k4arecord


class Record:
    def __init__(self, device_handle, device_configuration, filepath):
        self.record_handle = _k4arecord.k4a_record_t()
        self.header_written = False

        self.create_recording(device_handle, device_configuration, filepath)

    def __del__(self):
        self.flush()
        self.close()

    def create_recording(self, device_handle, device_configuration, filepath):
        _k4arecord.VERIFY(
            _k4arecord.k4a_record_create(
                filepath.encode("utf-8"),
                device_handle,
                device_configuration,
                self.record_handle,
            ),
            "Failed to create recording!",
        )

    def is_valid(self):
        return self.record_handle != None

    def close(self):
        if self.is_valid():
            _k4arecord.k4a_record_close(self.record_handle)
            self.record_handle = None
        else:
            print("sibal")

    def flush(self):
        if self.is_valid():
            _k4arecord.VERIFY(
                _k4arecord.k4a_record_flush(self.record_handle), "Failed to flush!"
            )

    def write_header(self):
        if self.is_valid():
            _k4arecord.VERIFY(
                _k4arecord.k4a_record_write_header(self.record_handle),
                "Failed to write header!",
            )

    def write_capture(self, capture_handle):
        if not self.is_valid():
            raise NameError("Recording not found")

        if not self.header_written:
            self.write_header()
            self.header_written = True
        _k4arecord.VERIFY(
            _k4arecord.k4a_record_write_capture(self.record_handle, capture_handle),
            "Failed to write capture!",
        )



import datetime
import logging
import os
import queue
import sys
import time
from typing import Optional, Tuple

import cv2
import numpy as np

import sounddevice as sd
import soundfile as sf

assert np  # avoid "imported but unused" message (W0611)

import pyk4a
from pydub import AudioSegment
from pyk4a import Config, ImageFormat, PyK4A, PyK4ARecord

from PySide6.QtCore import Qt, QThread, Signal, Slot
from PySide6.QtGui import QFont, QImage, QPixmap
from PySide6.QtWidgets import (
    QApplication,
    QDialog,
    QHBoxLayout,
    QLabel,
    QMainWindow,
    QPushButton,
    QSizePolicy,
    QVBoxLayout,
    QWidget,
)


CONFIG = Config(
    color_format=ImageFormat.COLOR_BGRA32,
    depth_mode=pyk4a.DepthMode.NFOV_UNBINNED,
    color_resolution=pyk4a.ColorResolution.RES_720P,
    synchronized_images_only=True,
)

# make output dir
# if not os.path.exists('./tools/outputs/'):
#     os.mkdir('./tools/outputs')


class RecordRGBD:
    pass


class Thread(QThread):
    rgb_updated_frame = Signal(QImage)
    depth_updated_frame = Signal(QImage)

    def __init__(self, parent=None):
        QThread.__init__(self, parent)
        self.status = True
        self.set_filename()

    def set_filename(self) -> None:
        filename = datetime.datetime.now()
        filename = filename.strftime("%Y_%m_%d_%H_%M_%S")
        # base_path = 'C:\\Program Files\\Azure Kinect SDK v1.4.1\\tools\\outputs'
        base_path = "C:\\Users\\qhdrm\\Videos"

        self.filename_video = f"{base_path}\\{filename}.mkv"
        self.filename_audio = f"{base_path}\\{filename}.wav"
        print(filename)

    def colorize(
        self,
        image: np.ndarray,
        clipping_range: Tuple[Optional[int], Optional[int]] = (None, None),
        colormap: int = cv2.COLORMAP_HSV,
    ) -> np.ndarray:
        if clipping_range[0] or clipping_range[1]:
            img = image.clip(clipping_range[0], clipping_range[1])  # type: ignore
        else:
            img = image.copy()
        img = cv2.normalize(img, None, 0, 255, cv2.NORM_MINMAX, dtype=cv2.CV_8U)
        img = cv2.applyColorMap(img, colormap)
        return img

    def run(self):
        config = CONFIG
        azure_device = PyK4A(config=config, device_id=0)
        azure_device.start()
        record = PyK4ARecord(
            device=azure_device, config=config, path=self.filename_video
        )
        record.create()
        print(self.status)
        while self.status:
            current_frame = azure_device.get_capture()
            record.write_capture(current_frame)

            if np.any(current_frame.color):
                color_frame = current_frame.color[:, :, :3].copy()
                color_frame = cv2.cvtColor(color_frame, cv2.COLOR_BGR2RGB)

                h, w, ch = color_frame.shape
                img = QImage(color_frame, w, h, ch * w, QImage.Format_RGB888)
                scaled_img = img.scaled(640, 480, Qt.KeepAspectRatio)
                self.rgb_updated_frame.emit(scaled_img)

            if np.any(current_frame.depth):
                depth_frame = self.colorize(
                    current_frame.depth, (None, 5000), cv2.COLORMAP_HSV
                )
                h, w, ch = depth_frame.shape

                depth_frame = QImage(depth_frame, w, h, w * ch, QImage.Format_RGB888)
                scaled_depth_frame = depth_frame.scaled(640, 480, Qt.KeepAspectRatio)
                self.depth_updated_frame.emit(scaled_depth_frame)


class MainWindow(QMainWindow):
    def __init__(self) -> None:
        super().__init__()
        self.logger = logging.getLogger()
        self.logger.setLevel(logging.DEBUG)

        fileHandler = logging.FileHandler("tools/pyk4a/example/outputs/log.txt")
        formatter = logging.Formatter(
            "[%(levelname)s] [%(asctime)s] (%(filename)s:%(lineno)d) > %(message)s"
        )
        fileHandler.setFormatter(formatter)
        self.logger.addHandler(fileHandler)

        self.config = CONFIG
        self.azure_device = PyK4A(config=self.config, device_id=0)
        self.record_flag = True

        # check record env
        if not self.initial_check():
            modal = QDialog()
            modal_layout = QVBoxLayout()

            e_message = QLabel("카메라에 문제가 있습니다. 재연결을 시도해주세요.")
            e_message.setAlignment(Qt.AlignCenter)
            e_message.setFont(QFont("Arial", 15))

            modal_layout.addWidget(e_message)
            modal.setLayout(modal_layout)
            modal.setWindowTitle("Error Message")
            modal.resize(400, 200)
            modal.exec()
            sys.exit(0)

        # initailize main window ui
        self.initial_window()

    def initial_check(self) -> bool:
        # self.logger
        initial_flag = True
        self.logger.debug("---------------녹화 시작 전 테스트를 진행합니다.---------------")
        self.logger.debug(
            f"\n \
            FPS : {str(self.azure_device._config.camera_fps)}\n \
            color_format: {str(self.azure_device._config.color_format)}\n \
            color_resolution: {str(self.azure_device._config.color_resolution)}\n \
            depth_mode: {str(self.azure_device._config.depth_mode)}"
        )
        self.logger.debug("\n---------------영상 테스트를 시작합니다.---------------")

        try:
            self.azure_device.start()
            num_frames = 0

            while num_frames < 10:
                frame = self.azure_device.get_capture()
                if frame.color.shape[2] != 4:
                    self.logger.debug("RGBD 영상의 차원이 올바르지 않습니다.")
                    initial_flag = False
                if not np.any(frame.depth):
                    self.logger.debug("Depth 영상을 찾을 수 없습니다.")
                    initial_flag = False

                num_frames += 1
            self.azure_device.close()

        except Exception as e:
            self.logger.debug(e)

        finally:
            self.logger.debug("카메라 연결 테스트를 종료합니다.")
            return initial_flag

    def initial_window(self) -> None:
        # Title and dimensions
        self.setWindowTitle("영유아 녹화 프로그램")
        self.setGeometry(0, 0, 1400, 600)

        # image layout
        images_layout = QHBoxLayout()
        self.time_label = QLabel(self)
        self.time_label.setAlignment(Qt.AlignCenter)
        self.time_label.setFont(QFont("Arial", 15))

        self.rgb_label = QLabel(self)
        self.rgb_label.setFixedSize(640, 480)
        self.depth_label = QLabel(self)
        self.depth_label.setFixedSize(640, 480)
        images_layout.addWidget(self.rgb_label)
        images_layout.addWidget(self.depth_label)

        # Thread in charge of updating the image
        self.th = Thread(self)
        # self.th.finished.connect(self.close)
        self.th.rgb_updated_frame.connect(self.setRGBImage)
        self.th.depth_updated_frame.connect(self.setDepthImage)

        # Buttons layout
        buttons_layout = QHBoxLayout()
        self.button1 = QPushButton("녹화 시작")
        self.button2 = QPushButton("녹화 종료")
        self.button1.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        self.button2.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        buttons_layout.addWidget(self.button1)
        buttons_layout.addWidget(self.button2)

        # Main layout
        layout = QVBoxLayout()
        layout.addWidget(self.time_label)
        layout.addLayout(images_layout)
        layout.addLayout(buttons_layout)

        # Central widget
        widget = QWidget(self)
        widget.setLayout(layout)
        self.setCentralWidget(widget)

        # Connections
        self.button1.clicked.connect(self.start)
        self.button2.clicked.connect(self.stop)
        self.button1.setEnabled(True)
        self.button2.setEnabled(False)

    @Slot()
    def stop(self) -> None:
        print("Finishing...")
        self.button2.setEnabled(False)
        self.button1.setEnabled(True)
        # cv2.destroyAllWindows()
        self.th.status = False
        # Give time for the thread to finish
        time.sleep(1)

    @Slot()
    def start(self) -> None:
        print("Starting...")
        self.button2.setEnabled(True)
        self.button1.setEnabled(False)
        self.th.set_filename()
        self.th.status = True
        self.th.start()

    @Slot(QImage)
    def setRGBImage(self, image: QImage) -> None:
        self.rgb_label.setPixmap(QPixmap.fromImage(image))

    @Slot(QImage)
    def setDepthImage(self, image: QImage):
        self.depth_label.setPixmap(QPixmap.fromImage(image))


if __name__ == "__main__":
    app = QApplication()
    w = MainWindow()
    w.show()
    sys.exit(app.exec())

    # main(args)



from PySide6.QtCore import Qt, Slot
from PySide6.QtWidgets import QFrame, QVBoxLayout, QHBoxLayout

from ..common_widgets import Label


class ImuSensors(QFrame):
    def __init__(self) -> None:
        super().__init__()
        self.setMaximumWidth(320)
        self.setMaximumHeight(480)

        self.setObjectName("IMUSensor")
        self.setStyleSheet(
            " QFrame#IMUSensor { border-color: white; }"
        )

        self.layout_main = QVBoxLayout()
        self.title = Label("IMU Sensor", orientation=Qt.AlignmentFlag.AlignCenter)
        self.title.setFixedHeight(60)

        self.layout_speed = QHBoxLayout()
        self.label_time = Label("Time(s) : --- ")
        self.label_time.setFixedHeight(30)
        self.label_fps = Label("FPS : ")
        self.label_fps.setFixedHeight(30)
        self.layout_speed.addWidget(self.label_time)
        self.layout_speed.addWidget(self.label_fps)

        self.layout_acc = QVBoxLayout()
        self.acc_title = Label("Accelerometer")
        self.acc_x = Label("X : ")
        self.acc_y = Label("Y : ")
        self.acc_z = Label("Z : ")

        self.layout_gyro = QVBoxLayout() 
        self.gyro_title = Label("Gyroscope")
        self.gyro_x = Label("X : ")
        self.gyro_y = Label("Y : ")
        self.gyro_z = Label("Z : ")

        self.layout_acc.addWidget(self.acc_title)
        self.layout_acc.addWidget(self.acc_x)
        self.layout_acc.addWidget(self.acc_y)
        self.layout_acc.addWidget(self.acc_z)

        self.layout_gyro.addWidget(self.gyro_title)
        self.layout_gyro.addWidget(self.gyro_x)
        self.layout_gyro.addWidget(self.gyro_y)
        self.layout_gyro.addWidget(self.gyro_z)

        self.layout_main.addWidget(self.title)
        self.layout_main.addLayout(self.layout_speed)
        self.layout_main.addWidget(self.label_time)
        self.layout_main.addLayout(self.layout_acc)
        self.layout_main.addLayout(self.layout_gyro)

        self.setLayout(self.layout_main)





import ctypes
import numpy as np

from pykinect_azure.k4a._k4atypes import k4a_float3_t, k4a_float2_t


# K4A_DECLARE_HANDLE(k4abt_tracker_t);
class _handle_k4abt_tracker_t(ctypes.Structure):
    _fields_ = [
        ("_rsvd", ctypes.c_size_t),
    ]


k4abt_tracker_t = ctypes.POINTER(_handle_k4abt_tracker_t)


# K4A_DECLARE_HANDLE(k4abt_frame_t);
class _handle_k4abt_frame_t(ctypes.Structure):
    _fields_ = [
        ("_rsvd", ctypes.c_size_t),
    ]


k4abt_frame_t = ctypes.POINTER(_handle_k4abt_frame_t)

k4abt_result_t = ctypes.c_int
K4ABT_RESULT_SUCCEEDED = 0
K4ABT_RESULT_FAILED = 1

# class k4abt_joint_id_t(CtypeIntEnum):
K4ABT_JOINT_PELVIS = 0
K4ABT_JOINT_SPINE_NAVEL = 1
K4ABT_JOINT_SPINE_CHEST = 2
K4ABT_JOINT_NECK = 3
K4ABT_JOINT_CLAVICLE_LEFT = 4
K4ABT_JOINT_SHOULDER_LEFT = 5
K4ABT_JOINT_ELBOW_LEFT = 6
K4ABT_JOINT_WRIST_LEFT = 7
K4ABT_JOINT_HAND_LEFT = 8
K4ABT_JOINT_HANDTIP_LEFT = 9
K4ABT_JOINT_THUMB_LEFT = 10
K4ABT_JOINT_CLAVICLE_RIGHT = 11
K4ABT_JOINT_SHOULDER_RIGHT = 12
K4ABT_JOINT_ELBOW_RIGHT = 13
K4ABT_JOINT_WRIST_RIGHT = 14
K4ABT_JOINT_HAND_RIGHT = 15
K4ABT_JOINT_HANDTIP_RIGHT = 16
K4ABT_JOINT_THUMB_RIGHT = 17
K4ABT_JOINT_HIP_LEFT = 18
K4ABT_JOINT_KNEE_LEFT = 19
K4ABT_JOINT_ANKLE_LEFT = 20
K4ABT_JOINT_FOOT_LEFT = 21
K4ABT_JOINT_HIP_RIGHT = 22
K4ABT_JOINT_KNEE_RIGHT = 23
K4ABT_JOINT_ANKLE_RIGHT = 24
K4ABT_JOINT_FOOT_RIGHT = 25
K4ABT_JOINT_HEAD = 26
K4ABT_JOINT_NOSE = 27
K4ABT_JOINT_EYE_LEFT = 28
K4ABT_JOINT_EAR_LEFT = 29
K4ABT_JOINT_EYE_RIGHT = 30
K4ABT_JOINT_EAR_RIGHT = 31
K4ABT_JOINT_COUNT = 32

K4ABT_JOINT_NAMES = [
    "pelvis",
    "spine - navel",
    "spine - chest",
    "neck",
    "left clavicle",
    "left shoulder",
    "left elbow",
    "left wrist",
    "left hand",
    " left handtip",
    "left thumb",
    "right clavicle",
    "right shoulder",
    "right elbow",
    "right wrist",
    "right hand",
    "right handtip",
    "right thumb",
    "left hip",
    "left knee",
    "left ankle",
    "left foot",
    "right hip",
    "right knee",
    "right ankle",
    "right foot",
    "head",
    "nose",
    "left eye",
    "left ear",
    "right eye",
    "right ear",
]

K4ABT_SEGMENT_PAIRS = [
    [1, 0],
    [2, 1],
    [3, 2],
    [4, 2],
    [5, 4],
    [6, 5],
    [7, 6],
    [8, 7],
    [9, 8],
    [10, 7],
    [11, 2],
    [12, 11],
    [13, 12],
    [14, 13],
    [15, 14],
    [16, 15],
    [17, 14],
    [18, 0],
    [19, 18],
    [20, 19],
    [21, 20],
    [22, 0],
    [23, 22],
    [24, 23],
    [25, 24],
    [26, 3],
    [27, 26],
    [28, 26],
    [29, 26],
    [30, 26],
    [31, 26],
]

# class k4abt_sensor_orientation_t(CtypeIntEnum):
K4ABT_SENSOR_ORIENTATION_DEFAULT = 0
K4ABT_SENSOR_ORIENTATION_CLOCKWISE90 = 1
K4ABT_SENSOR_ORIENTATION_COUNTERCLOCKWISE90 = 2
K4ABT_SENSOR_ORIENTATION_FLIP180 = 3

# class k4abt_tracker_processing_mode_t(CtypeIntEnum):
K4ABT_TRACKER_PROCESSING_MODE_GPU = 0
K4ABT_TRACKER_PROCESSING_MODE_CPU = 1
K4ABT_TRACKER_PROCESSING_MODE_GPU_CUDA = 2
K4ABT_TRACKER_PROCESSING_MODE_GPU_TENSORRT = 3
K4ABT_TRACKER_PROCESSING_MODE_GPU_DIRECTML = 4


class _k4abt_tracker_configuration_t(ctypes.Structure):
    _fields_ = [
        ("sensor_orientation", ctypes.c_int),
        ("processing_mode", ctypes.c_int),
        ("gpu_device_id", ctypes.c_int32),
        ("model_path", ctypes.c_char_p),
    ]


k4abt_tracker_configuration_t = _k4abt_tracker_configuration_t


class _wxyz(ctypes.Structure):
    _fields_ = [
        ("w", ctypes.c_float),
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("z", ctypes.c_float),
    ]

    def __iter__(self):
        return {"w": self.w, "x": self.x, "y": self.y, "z": self.z}

    def __str__(self):
        return f"w:{self.w} x:{self.x} y:{self.y} z:{self.z}"


class k4a_quaternion_t(ctypes.Union):
    _fields_ = [("wxyz", _wxyz), ("v", ctypes.c_float * 4)]

    def __init__(self, q=(0, 0, 0, 0)):
        super().__init__()
        self.wxyz = _wxyz(q[0], q[1], q[2], q[3])

    def __iter__(self):
        wxyz = self.wxyz.__iter__()
        wxyz.update({"v": [v for v in self.v]})
        return wxyz

    def __str__(self):
        return self.wxyz.__str__()


# class k4abt_joint_confidence_level_t(CtypeIntEnum):
K4ABT_JOINT_CONFIDENCE_NONE = 0
K4ABT_JOINT_CONFIDENCE_LOW = 1
K4ABT_JOINT_CONFIDENCE_MEDIUM = 2
K4ABT_JOINT_CONFIDENCE_HIGH = 3
K4ABT_JOINT_CONFIDENCE_LEVELS_COUNT = 4


class _k4abt_joint_t(ctypes.Structure):
    _fields_ = [
        ("position", k4a_float3_t),
        ("orientation", k4a_quaternion_t),
        ("confidence_level", ctypes.c_int),
    ]

    def __init__(
        self, position=(0, 0, 0), orientation=(0, 0, 0, 0), confidence_level=0
    ):
        super().__init__()
        self.position = k4a_float3_t(position)
        self.orientation = k4a_quaternion_t(orientation)
        self.confidence_level = confidence_level

    def __iter__(self):
        return {
            "position": self.position.__iter__(),
            "orientation": self.orientation.__iter__(),
            "confidence_level": self.confidence_level,
        }


k4abt_joint_t = _k4abt_joint_t


class k4abt_skeleton_t(ctypes.Structure):
    _fields_ = [
        ("joints", _k4abt_joint_t * K4ABT_JOINT_COUNT),
    ]

    def __init__(self, joints=(_k4abt_joint_t() for i in range(K4ABT_JOINT_COUNT))):
        super().__init__()
        self.joints = (_k4abt_joint_t * K4ABT_JOINT_COUNT)(*joints)

    def __iter__(self):
        return {"joints": [joint.__iter__() for joint in self.joints]}


class k4abt_body_t(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("skeleton", k4abt_skeleton_t),
    ]

    def __init__(self, id=0, skeleton=k4abt_skeleton_t()):
        super().__init__()
        self.id = id
        self.skeleton = skeleton

    def __iter__(self):
        return {"id": self.id, "skeleton": self.skeleton.__iter__()}


class _k4abt_joint2D_t(ctypes.Structure):
    _fields_ = [
        ("position", k4a_float2_t),
        ("confidence_level", ctypes.c_int),
    ]

    def __init__(self, position=(0, 0), confidence_level=0):
        super().__init__()
        self.position = k4a_float2_t(position)
        self.confidence_level = confidence_level

    def __iter__(self):
        return {
            "position": self.position.__iter__(),
            "confidence_level": self.confidence_level,
        }


k4abt_joint2D_t = _k4abt_joint2D_t


class k4abt_skeleton2D_t(ctypes.Structure):
    _fields_ = [
        ("joints2D", _k4abt_joint2D_t * K4ABT_JOINT_COUNT),
    ]

    def __init__(self, joints=(_k4abt_joint2D_t() for i in range(K4ABT_JOINT_COUNT))):
        super().__init__()
        self.joints2D = (_k4abt_joint2D_t * K4ABT_JOINT_COUNT)(*joints)

    def __iter__(self):
        return {"joints2D": [joint.__iter__() for joint in self.joints2D]}


class k4abt_body2D_t(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("skeleton", k4abt_skeleton2D_t),
    ]

    def __init__(self, id=0, skeleton=k4abt_skeleton2D_t()):
        super().__init__()
        self.id = id
        self.skeleton = skeleton

    def __iter__(self):
        return {"id": self.id, "skeleton": self.skeleton.__iter__()}


K4ABT_BODY_INDEX_MAP_BACKGROUND = 255
K4ABT_INVALID_BODY_ID = 0xFFFFFFFF
K4ABT_DEFAULT_TRACKER_SMOOTHING_FACTOR = 0.0

K4ABT_DEFAULT_MODEL = 0
K4ABT_LITE_MODEL = 1

k4abt_tracker_default_configuration = k4abt_tracker_configuration_t()
k4abt_tracker_default_configuration.sensor_orientation = (
    K4ABT_SENSOR_ORIENTATION_DEFAULT
)
k4abt_tracker_default_configuration.processing_mode = K4ABT_TRACKER_PROCESSING_MODE_GPU
k4abt_tracker_default_configuration.gpu_device_id = 0

body_colors = np.ones((256, 3), dtype=np.uint8) * K4ABT_BODY_INDEX_MAP_BACKGROUND
body_colors[:7, :] = np.array(
    [
        [202, 183, 42],
        [42, 61, 202],
        [42, 202, 183],
        [202, 42, 61],
        [183, 42, 202],
        [42, 202, 61],
        [141, 202, 42],
    ]
)



import ctypes
import sys
import traceback

from ._k4arecordTypes import *
from ..k4a._k4atypes import *

record_dll = None


def setup_library(module_k4arecord_path):
    global record_dll

    try:
        record_dll = ctypes.CDLL(module_k4arecord_path)
    except Exception as e:
        print("Failed to load library", e)
        sys.exit(1)


def k4a_record_create(file_path, device, device_config, recording_handle):
    """
    K4ARECORD_EXPORT k4a_result_t k4a_record_create(
        const char *path,
        k4a_device_t device,
        const k4a_device_configuration_t device_config,
        k4a_record_t *recording_handle
    );
    """
    _k4a_record_create = record_dll.k4a_record_create
    _k4a_record_create.restype = k4a_result_t
    _k4a_record_create.argtypes = (
        ctypes.POINTER(ctypes.c_char),
        k4a_device_t,
        k4a_device_configuration_t,
        ctypes.POINTER(k4a_record_t),
    )

    return _k4a_record_create(file_path, device, device_config, recording_handle)


def k4a_record_write_header(recording_handle):
    # K4ARECORD_EXPORT k4a_result_t k4a_record_write_header(k4a_record_t recording_handle);
    _k4a_record_write_header = record_dll.k4a_record_write_header
    _k4a_record_write_header.restype = k4a_result_t
    _k4a_record_write_header.argtypes = (k4a_record_t,)

    return _k4a_record_write_header(recording_handle)


def k4a_record_write_capture(recording_handle, capture_handle):
    # K4ARECORD_EXPORT k4a_result_t k4a_record_write_capture(k4a_record_t recording_handle, k4a_capture_t capture_handle);
    _k4a_record_write_capture = record_dll.k4a_record_write_capture
    _k4a_record_write_capture.restype = k4a_result_t
    _k4a_record_write_capture.argtypes = (k4a_record_t, k4a_capture_t)

    return _k4a_record_write_capture(recording_handle, capture_handle)


def k4a_record_flush(recording_handle):
    # K4ARECORD_EXPORT k4a_result_t k4a_record_flush(k4a_record_t recording_handle);
    _k4a_record_flush = record_dll.k4a_record_flush
    _k4a_record_flush.restype = k4a_result_t
    _k4a_record_flush.argtypes = (k4a_record_t,)

    return _k4a_record_flush(recording_handle)


def k4a_record_close(recording_handle):
    # K4ARECORD_EXPORT void k4a_record_close(k4a_record_t recording_handle);
    _k4a_record_close = record_dll.k4a_record_close
    _k4a_record_close.restype = None
    _k4a_record_close.argtypes = (k4a_record_t,)

    _k4a_record_close(recording_handle)


###########################
###    Playback         ###
###########################


def k4a_playback_open(file_path, playback_handle):
    # K4ARECORD_EXPORT k4a_result_t k4a_playback_open(const char *path, k4a_playback_t *playback_handle);
    _k4a_playback_open = record_dll.k4a_playback_open
    _k4a_playback_open.restype = k4a_result_t
    _k4a_playback_open.argtypes = (
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(k4a_playback_t),
    )

    return _k4a_playback_open(file_path, playback_handle)


def k4a_playback_close(playback_handle):
    # K4ARECORD_EXPORT void k4a_playback_close(k4a_playback_t playback_handle);
    _k4a_playback_close = record_dll.k4a_playback_close
    _k4a_playback_close.restype = None
    _k4a_playback_close.argtypes = (k4a_playback_t,)

    _k4a_playback_close(playback_handle)

# get vs get raw?
def k4a_playback_get_raw_calibration(playback_handle, data, data_size):
    """
    K4ARECORD_EXPORT k4a_buffer_result_t k4a_playback_get_raw_calibration(
        k4a_playback_t playback_handle,
        uint8_t *data,
        size_t *data_size
    );
    """
    _k4a_playback_get_raw_calibration = record_dll.k4a_playback_get_raw_calibration
    _k4a_playback_get_raw_calibration.restype = k4a_buffer_result_t
    _k4a_playback_get_raw_calibration.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_uint8),
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_playback_get_raw_calibration(playback_handle, data, data_size)


def k4a_playback_get_calibration(playback_handle, calibration):
    """
    K4ARECORD_EXPORT k4a_result_t k4a_playback_get_calibration(
        k4a_playback_t playback_handle,
        k4a_calibration_t *calibration
    );
    """
    _k4a_playback_get_calibration = record_dll.k4a_playback_get_calibration
    _k4a_playback_get_calibration.restype = k4a_result_t
    _k4a_playback_get_calibration.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(k4a_calibration_t),
    )

    return _k4a_playback_get_calibration(playback_handle, calibration)


def k4a_playback_get_record_configuration(playback_handle, config):
    """
    K4ARECORD_EXPORT k4a_result_t k4a_playback_get_record_configuration(
        k4a_playback_t playback_handle,
        k4a_record_configuration_t *config
    );
    """
    _k4a_playback_get_record_configuration = (
        record_dll.k4a_playback_get_record_configuration
    )
    _k4a_playback_get_record_configuration.restype = k4a_result_t
    _k4a_playback_get_record_configuration.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(k4a_record_configuration_t),
    )

    return _k4a_playback_get_record_configuration(playback_handle, config)

# Check for files with the same name
def k4a_playback_check_track_exists(playback_handle, track_name):
    """
    K4ARECORD_EXPORT bool k4a_playback_check_track_exists(
        k4a_playback_t playback_handle, 
        const char *track_name
    );
    """
    _k4a_playback_check_track_exists = record_dll.k4a_playback_check_track_exists
    _k4a_playback_check_track_exists.restype = ctypes.c_bool
    _k4a_playback_check_track_exists.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
    )

    return _k4a_playback_check_track_exists(playback_handle, track_name)


def k4a_playback_get_track_count(playback_handle):
    """
    K4ARECORD_EXPORT size_t k4a_playback_get_track_count(k4a_playback_t playback_handle);
    """
    _k4a_playback_get_track_count = record_dll.k4a_playback_get_track_count
    _k4a_playback_get_track_count.restype = ctypes.c_size_t
    _k4a_playback_get_track_count.argtypes = (k4a_playback_t,)

    return _k4a_playback_get_track_count(playback_handle)


#extract video with filename 
def k4a_playback_get_track_name(
    playback_handle, track_index, track_name, track_name_size
):
    """
    K4ARECORD_EXPORT k4a_buffer_result_t k4a_playback_get_track_name(
        k4a_playback_t playback_handle,
        size_t track_index,
        char *track_name,
        size_t *track_name_size
    );
    """
    _k4a_playback_get_track_name = record_dll.k4a_playback_get_track_name
    _k4a_playback_get_track_name.restype = k4a_buffer_result_t
    _k4a_playback_get_track_name.argtypes = (
        k4a_playback_t,
        ctypes.c_size_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_playback_get_track_name(
        playback_handle, track_index, track_name, track_name_size
    )


def k4a_playbk4a_playback_track_is_builtinack_get_track_name(
    playback_handle, track_name
):
    """
    K4ARECORD_EXPORT bool k4a_playback_track_is_builtin(
        k4a_playback_t playback_handle, 
        const char *track_name
    );
    """
    _k4a_playback_track_is_builtin = record_dll.k4a_playback_track_is_builtin
    _k4a_playback_track_is_builtin.restype = ctypes.c_bool
    _k4a_playback_track_is_builtin.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
    )

    return _k4a_playback_track_is_builtin(playback_handle, track_name)


def k4a_playback_track_get_video_settings(playback_handle, track_name, video_settings):
    """
    K4ARECORD_EXPORT k4a_result_t k4a_playback_track_get_video_settings(
        k4a_playback_t playback_handle,
        const char *track_name,
        k4a_record_video_settings_t *video_settings
    );
    """
    _k4a_playback_track_get_video_settings = (
        record_dll.k4a_playback_track_get_video_settings
    )
    _k4a_playback_track_get_video_settings.restype = k4a_result_t
    _k4a_playback_track_get_video_settings.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(k4a_record_video_settings_t),
    )

    return _k4a_playback_track_get_video_settings(
        playback_handle, track_name, video_settings
    )


def k4a_playback_track_get_codec_id(
    playback_handle, track_name, codec_id, codec_id_size
):
    """
    K4ARECORD_EXPORT k4a_buffer_result_t k4a_playback_track_get_codec_id(
        k4a_playback_t playback_handle,
        const char *track_name,
        char *codec_id,
        size_t *codec_id_size
    );
    """
    _k4a_playback_track_get_codec_id = record_dll.k4a_playback_track_get_codec_id
    _k4a_playback_track_get_codec_id.restype = k4a_buffer_result_t
    _k4a_playback_track_get_codec_id.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_playback_track_get_codec_id(
        playback_handle, track_name, codec_id, codec_id_size
    )


def k4a_playback_track_get_codec_context(
    playback_handle, track_name, codec_context, codec_context_size
):
    """
    K4ARECORD_EXPORT k4a_buffer_result_t k4a_playback_track_get_codec_context(
        k4a_playback_t playback_handle,
        const char *track_name,
        uint8_t *codec_context,
        size_t *codec_context_size
    );
    """
    _k4a_playback_track_get_codec_context = (
        record_dll.k4a_playback_track_get_codec_context
    )
    _k4a_playback_track_get_codec_context.restype = k4a_buffer_result_t
    _k4a_playback_track_get_codec_context.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(ctypes.c_uint8),
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_playback_track_get_codec_context(
        playback_handle, track_name, codec_context, codec_context_size
    )


def k4a_playback_get_tag(playback_handle, name, value, value_size):
    """
    K4ARECORD_EXPORT k4a_buffer_result_t k4a_playback_get_tag(
        k4a_playback_t playback_handle,
        const char *name,
        char *value,
        size_t *value_size
    );
    """
    _k4a_playback_get_tag = record_dll.k4a_playback_get_tag
    _k4a_playback_get_tag.restype = k4a_buffer_result_t
    _k4a_playback_get_tag.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_playback_get_tag(playback_handle, name, value, value_size)


# convert 
def k4a_playback_set_color_conversion(playback_handle, target_format):
    """
    K4ARECORD_EXPORT k4a_result_t k4a_playback_set_color_conversion(
        k4a_playback_t playback_handle,
        k4a_image_format_t target_format
    );
    """
    _k4a_playback_set_color_conversion = record_dll.k4a_playback_set_color_conversion
    _k4a_playback_set_color_conversion.restype = k4a_result_t
    _k4a_playback_set_color_conversion.argtypes = (k4a_playback_t, k4a_image_format_t)

    return _k4a_playback_set_color_conversion(playback_handle, target_format)


def k4a_playback_get_attachment(playback_handle, file_name, data, data_size):
    """
    K4ARECORD_EXPORT k4a_buffer_result_t k4a_playback_get_attachment(
        k4a_playback_t playback_handle,
        const char *file_name,
        uint8_t *data,
        size_t *data_size
    );
    """
    _k4a_playback_get_attachment = record_dll.k4a_playback_get_attachment
    _k4a_playback_get_attachment.restype = k4a_buffer_result_t
    _k4a_playback_get_attachment.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(ctypes.c_uint8),
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_playback_get_attachment(playback_handle, file_name, data, data_size)


def k4a_playback_get_next_capture(playback_handle, capture_handle):
    """
    K4ARECORD_EXPORT k4a_stream_result_t k4a_playback_get_next_capture(
        k4a_playback_t playback_handle,
        k4a_capture_t *capture_handle
    );
    """
    _k4a_playback_get_next_capture = record_dll.k4a_playback_get_next_capture
    _k4a_playback_get_next_capture.restype = k4a_stream_result_t
    _k4a_playback_get_next_capture.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(k4a_capture_t),
    )

    return _k4a_playback_get_next_capture(playback_handle, capture_handle)


def k4a_playback_get_previous_capture(playback_handle, capture_handle):
    """
    K4ARECORD_EXPORT k4a_stream_result_t k4a_playback_get_previous_capture(
        k4a_playback_t playback_handle,
        k4a_capture_t *capture_handle
    );
    """
    _k4a_playback_get_previous_capture = record_dll.k4a_playback_get_previous_capture
    _k4a_playback_get_previous_capture.restype = k4a_stream_result_t
    _k4a_playback_get_previous_capture.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(k4a_capture_t),
    )

    return _k4a_playback_get_previous_capture(playback_handle, capture_handle)


def k4a_playback_get_next_imu_sample(playback_handle, imu_sample):
    """
    K4ARECORD_EXPORT k4a_stream_result_t k4a_playback_get_next_imu_sample(
        k4a_playback_t playback_handle,
        k4a_imu_sample_t *imu_sample
    );
    """
    _k4a_playback_get_next_imu_sample = record_dll.k4a_playback_get_next_imu_sample
    _k4a_playback_get_next_imu_sample.restype = k4a_stream_result_t
    _k4a_playback_get_next_imu_sample.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(k4a_imu_sample_t),
    )

    return _k4a_playback_get_next_imu_sample(playback_handle, imu_sample)


def k4a_playback_get_previous_imu_sample(playback_handle, imu_sample):
    """
    K4ARECORD_EXPORT k4a_stream_result_t k4a_playback_get_previous_imu_sample(
        k4a_playback_t playback_handle,
        k4a_imu_sample_t *imu_sample
    );
    """
    _k4a_playback_get_previous_imu_sample = (
        record_dll.k4a_playback_get_previous_imu_sample
    )
    _k4a_playback_get_previous_imu_sample.restype = k4a_stream_result_t
    _k4a_playback_get_previous_imu_sample.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(k4a_imu_sample_t),
    )

    return _k4a_playback_get_previous_imu_sample(playback_handle, imu_sample)


def k4a_playback_get_next_data_block(playback_handle, track_name, data_block_handle):
    """
    K4ARECORD_EXPORT k4a_stream_result_t k4a_playback_get_next_data_block(
        k4a_playback_t playback_handle,
        const char *track_name,
        k4a_playback_data_block_t *data_block_handle
    );
    """
    _k4a_playback_get_next_data_block = record_dll.k4a_playback_get_next_data_block
    _k4a_playback_get_next_data_block.restype = k4a_stream_result_t
    _k4a_playback_get_next_data_block.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(k4a_playback_data_block_t),
    )

    return _k4a_playback_get_next_data_block(
        playback_handle, track_name, data_block_handle
    )


def k4a_playback_get_previous_data_block(
    playback_handle, track_name, data_block_handle
):
    """
    K4ARECORD_EXPORT k4a_stream_result_t k4a_playback_get_previous_data_block(
        k4a_playback_t playback_handle,
        const char *track_name,
        k4a_playback_data_block_t *data_block_handle
    );
    """
    _k4a_playback_get_previous_data_block = (
        record_dll.k4a_playback_get_previous_data_block
    )
    _k4a_playback_get_previous_data_block.restype = k4a_stream_result_t
    _k4a_playback_get_previous_data_block.argtypes = (
        k4a_playback_t,
        ctypes.POINTER(ctypes.c_char),
        ctypes.POINTER(k4a_playback_data_block_t),
    )

    return _k4a_playback_get_previous_data_block(
        playback_handle, track_name, data_block_handle
    )


# Gets the last timestamp in a recording, relative to the start of the recording.
def k4a_playback_data_block_get_device_timestamp_usec(data_block_handle):
    """
    K4ARECORD_EXPORT uint64_t k4a_playback_data_block_get_device_timestamp_usec(
        k4a_playback_data_block_t data_block_handle
    );
    """
    _k4a_playback_data_block_get_device_timestamp_usec = (
        record_dll.k4a_playback_data_block_get_device_timestamp_usec
    )
    _k4a_playback_data_block_get_device_timestamp_usec.restype = ctypes.c_uint64
    _k4a_playback_data_block_get_device_timestamp_usec.argtypes = (
        k4a_playback_data_block_t,
    )

    return _k4a_playback_data_block_get_device_timestamp_usec(data_block_handle)


# 
def k4a_playback_data_block_get_buffer_size(data_block_handle):
    """
    K4ARECORD_EXPORT size_t k4a_playback_data_block_get_buffer_size(
        k4a_playback_data_block_t data_block_handle
    );
    """
    _k4a_playback_data_block_get_buffer_size = (
        record_dll.k4a_playback_data_block_get_buffer_size
    )
    _k4a_playback_data_block_get_buffer_size.restype = ctypes.c_size_t
    _k4a_playback_data_block_get_buffer_size.argtypes = (k4a_playback_data_block_t,)

    return _k4a_playback_data_block_get_buffer_size(data_block_handle)


def k4a_playback_data_block_get_buffer(data_block_handle):
    """
    K4ARECORD_EXPORT uint8_t *k4a_playback_data_block_get_buffer(
        k4a_playback_data_block_t data_block_handle
    );
    """
    _k4a_playback_data_block_get_buffer = record_dll.k4a_playback_data_block_get_buffer
    _k4a_playback_data_block_get_buffer.restype = ctypes.POINTER(ctypes.c_uint8)
    _k4a_playback_data_block_get_buffer.argtypes = (k4a_playback_data_block_t,)

    return _k4a_playback_data_block_get_buffer(data_block_handle)


def k4a_playback_data_block_release(data_block_handle):
    """
    K4ARECORD_EXPORT void k4a_playback_data_block_release(
        k4a_playback_data_block_t data_block_handle
    );
    """
    _k4a_playback_data_block_release = record_dll.k4a_playback_data_block_release
    _k4a_playback_data_block_release.restype = None
    _k4a_playback_data_block_release.argtypes = (k4a_playback_data_block_t,)

    return _k4a_playback_data_block_release(data_block_handle)


def k4a_playback_seek_timestamp(playback_handle, offset_usec, origin):
    """
    K4ARECORD_EXPORT k4a_result_t k4a_playback_seek_timestamp(
        k4a_playback_t playback_handle,
        int64_t offset_usec,
        k4a_playback_seek_origin_t origin
    );
    """
    _k4a_playback_seek_timestamp = record_dll.k4a_playback_seek_timestamp
    _k4a_playback_seek_timestamp.restype = k4a_result_t
    _k4a_playback_seek_timestamp.argtypes = (
        k4a_playback_t,
        ctypes.c_int64,
        k4a_playback_seek_origin_t,
    )

    return _k4a_playback_seek_timestamp(playback_handle, offset_usec, origin)


def k4a_playback_get_recording_length_usec(playback_handle):
    """
    K4ARECORD_EXPORT uint64_t k4a_playback_get_recording_length_usec(
        k4a_playback_t playback_handle
    );
    """
    _k4a_playback_get_recording_length_usec = (
        record_dll.k4a_playback_get_recording_length_usec
    )
    _k4a_playback_get_recording_length_usec.restype = ctypes.c_uint64
    _k4a_playback_get_recording_length_usec.argtypes = (k4a_playback_t,)

    return _k4a_playback_get_recording_length_usec(playback_handle)


def k4a_playback_get_last_timestamp_usec(playback_handle):
    """
    K4ARECORD_DEPRECATED_EXPORT uint64_t k4a_playback_get_last_timestamp_usec(
        k4a_playback_t playback_handle
    );
    """
    _k4a_playback_get_last_timestamp_usec = (
        record_dll.k4a_playback_get_last_timestamp_usec
    )
    _k4a_playback_get_last_timestamp_usec.restype = ctypes.c_uint64
    _k4a_playback_get_last_timestamp_usec.argtypes = (k4a_playback_t,)

    return _k4a_playback_get_last_timestamp_usec(playback_handle)


def VERIFY(result, error):
    if result != K4A_RESULT_SUCCEEDED:
        print(error)
        traceback.print_stack()
        sys.exit(1)



from PySide6.QtGui import QPainter, QPen, QColor
from PySide6.QtWidgets import (
    QHBoxLayout, QLabel, QVBoxLayout, QRadioButton,
    QWidget, QGridLayout, QCheckBox, QPushButton, QFrame
)
from PySide6.QtCore import Qt, QRect
from ..common_widgets import ComboBox, Slider
from ..signals import all_signals


class ViewerSidebar(QFrame):
    def __init__(self) -> None:
        super().__init__()
        self.setStyleSheet("background-color: #252526;")

        layout_main = QVBoxLayout()
        layout_main.addWidget(RgbCameraPanel())
        layout_main.addWidget(DepthCameraPanel())
        layout_main.addWidget(IRCameraPanel())
        layout_main.addWidget(AudioPanel())
        self.setLayout(layout_main)

        self.setMaximumHeight(1080)
        self.setMaximumWidth(300)


class RgbCameraPanel(QFrame):
    def __init__(self) -> None:
        super().__init__()
        # 메인레이아웃
        self.setObjectName("RgbCameraPanel")
        self.setFixedHeight(550)
        self.setStyleSheet("""
            QFrame#RgbCameraPanel {
                border-color: gray; border-width: 2px; border-radius: 5px;
            }
        """)

        layout_main = QVBoxLayout()
        layout_title = QHBoxLayout()

        # toggle btn
        self.is_change = True
        self.btn_switch = ToggleButton()
        self.btn_switch.clicked.connect(self._toggle)
        layout_title.addWidget(QLabel("<b>RGB Camera Option<b>"))
        layout_title.addWidget(QLabel(""))
        layout_title.addWidget(self.btn_switch)

        layout_grid = QGridLayout()
        layout_grid.addWidget(QLabel("Resolution"), 0, 0)
        self.btn_resolutions = ComboBox(
            ["720p", "1080p", "1440p", "1536p", "2160p", "3072p"], 0
        )
        layout_grid.addWidget(self.btn_resolutions, 0, 1)
        self.btn_resolutions.currentIndexChanged.connect(self.set_config)

        layout_grid.addWidget(QLabel("Color Format"), 1, 0)
        self.btn_rgbformat = ComboBox(
            ["MJPG", "NV12", "YUV2", "BGRA"], 3
        )
        layout_grid.addWidget(self.btn_rgbformat, 1, 1)
        self.btn_rgbformat.currentIndexChanged.connect(self.set_config)

        layout_grid.addWidget(QLabel("FPS"), 2, 0)
        self.btn_fps = ComboBox(
            ["5", "15", "30"], 2
        )
        layout_grid.addWidget(self.btn_fps, 2, 1)
        self.btn_fps.currentIndexChanged.connect(self.set_config)

        self.control_panel = ColorControlPanel()
        layout_main.addLayout(layout_title)
        layout_main.addLayout(layout_grid)
        layout_main.addWidget(self.control_panel)

        self.set_config()
        self.setLayout(layout_main)
        
    def _toggle(self) -> None:
        if self.is_change:
            self.btn_resolutions.setDisabled(True)
            self.btn_rgbformat.setDisabled(True)
            self.btn_fps.setDisabled(True)
            self.control_panel.setDisabled(True)
            self.is_change = False
        else:
            self.btn_resolutions.setDisabled(False)
            self.btn_rgbformat.setDisabled(False)
            self.btn_fps.setDisabled(False)
            self.control_panel.setDisabled(False)
            self.is_change = True
            
    def set_config(self) -> None:
        color = {
            "color_resolution": self.btn_resolutions.currentIndex()+1,
            "color_format": self.btn_rgbformat.currentIndex(),
            "camera_fps": self.btn_fps.currentIndex()
        }
        _config_sidebar["color"] = color
        all_signals.config_viewer.emit(_config_sidebar)
    

class ColorControlPanel(QWidget):
    def __init__(self) -> None:
        super().__init__()
        # 메인 레이아웃
        layout = QGridLayout()

        layout.addWidget(QLabel("Exposure Time"), 0, 0)
        self.exposure_time = Slider(Qt.Orientation.Horizontal, (500, 1000000), 33300)
        layout.addWidget(self.exposure_time, 0, 1)
        self.exposure_time.sliderReleased.connect(self.set_config)

        layout.addWidget(QLabel("White Balance"), 1, 0)
        self.white_balance = Slider(Qt.Orientation.Horizontal, (2500, 12500), 4500)
        layout.addWidget(self.white_balance, 1, 1)
        self.white_balance.sliderReleased.connect(self.set_config)

        layout.addWidget(QLabel("Brightness"), 2, 0)
        self.brightness = Slider(Qt.Orientation.Horizontal, (0, 255), 128)
        layout.addWidget(self.brightness, 2, 1)
        self.brightness.sliderReleased.connect(self.set_config)

        layout.addWidget(QLabel("Contrast"), 3, 0)
        self.contrast = Slider(Qt.Orientation.Horizontal, (0, 10), 5)
        layout.addWidget(self.contrast, 3, 1)
        self.contrast.sliderReleased.connect(self.set_config)

        layout.addWidget(QLabel("Saturation"), 4, 0)
        self.saturation = Slider(Qt.Orientation.Horizontal, (0, 63), 32)
        layout.addWidget(self.saturation, 4, 1)
        self.saturation.sliderReleased.connect(self.set_config)

        layout.addWidget(QLabel("Sharpness"), 5, 0)
        self.sharpness = Slider(Qt.Orientation.Horizontal, (0, 4), 2)
        layout.addWidget(self.sharpness, 5, 1)
        self.sharpness.sliderReleased.connect(self.set_config)

        layout.addWidget(QLabel("Gain"), 6, 0)
        self.gain = Slider(Qt.Orientation.Horizontal, (0, 255), 128)
        layout.addWidget(self.gain, 6, 1)
        self.gain.sliderReleased.connect(self.set_config)

        self.backlight = QCheckBox("Backlight compensation")
        self.backlight.setCheckable(True)
        self.backlight.stateChanged.connect(self.set_config)
        layout.addWidget(self.backlight, 7, 0, 1, 2)

        layout.addWidget(QLabel("Power Freq"), 8, 0)
        layout_power_freq = QHBoxLayout()
        self.power_freq1 = QRadioButton("50hz")
        self.power_freq2 = QRadioButton("60hz")
        self.power_freq2.setChecked(True)
        self.power_freq1.clicked.connect(self.set_config)
        self.power_freq2.clicked.connect(self.set_config)
        layout_power_freq.addWidget(self.power_freq1)
        layout_power_freq.addWidget(self.power_freq2)
        layout.addLayout(layout_power_freq, 8, 1)

        self.set_config()
        self.setLayout(layout)

    def set_config(self) -> None:
        color_option = {
            "K4A_COLOR_CONTROL_EXPOSURE_TIME_ABSOLUTE": self.exposure_time.value(),
            "K4A_COLOR_CONTROL_WHITEBALANCE": self.white_balance.value(),
            "K4A_COLOR_CONTROL_CONTRAST": self.contrast.value(),
            "K4A_COLOR_CONTROL_SATURATION": self.saturation.value(),
            "K4A_COLOR_CONTROL_SHARPNESS": self.sharpness.value(),
            "K4A_COLOR_CONTROL_BRIGHTNESS": self.brightness.value(),
            "K4A_COLOR_CONTROL_GAIN": self.gain.value(),
            "K4A_COLOR_CONTROL_BACKLIGHT_COMPENSATION": self.backlight.isChecked(),
            "K4A_COLOR_CONTROL_POWERLINE_FREQUENCY": "1" if self.power_freq1.isChecked() else "2"
        }
        _config_sidebar["color_option"] = color_option
        all_signals.config_viewer.emit(_config_sidebar)

   
class DepthCameraPanel(QFrame):
    def __init__(self) -> None:
        super().__init__()
        # 메인레이아웃
        self.setObjectName("DepthCameraPanel")
        self.setStyleSheet("""
            QFrame#DepthCameraPanel {
                border-color: gray; border-width: 2px; border-radius: 5px;
            }
        """)

        layout_grid = QGridLayout()
        layout_title = QHBoxLayout()
        self.is_change = True

        layout_grid.addWidget(QLabel("<b>Depth Camera Option<b>"), 0, 0)
        self.btn_switch = ToggleButton()
        self.btn_switch.clicked.connect(self._toggle)
        layout_title.addWidget(QLabel(""))
        layout_title.addWidget(self.btn_switch)
        layout_grid.addLayout(layout_title, 0, 1)

        layout_grid.addWidget(QLabel("Depth mode"), 1, 0)
        self.btn_depth = ComboBox(
            ["NFOV_Binned", "NFOV_Unbinned", "WFOV_Binned", "WFOV_UnBinned", "PASSIVE_IR"], 1
        )
        layout_grid.addWidget(self.btn_depth, 1, 1)
        self.btn_depth.currentIndexChanged.connect(self.set_config)

        layout_grid.addWidget(QLabel("FPS"), 2, 0)
        self.btn_fps = ComboBox(
            ["5", "15", "30"], 2
        )
        layout_grid.addWidget(self.btn_fps, 2, 1)
        self.btn_fps.currentIndexChanged.connect(self.set_config)

        self.set_config()
        self.setLayout(layout_grid)

    def _toggle(self) -> None:
        if self.is_change:
            self.btn_depth.setDisabled(True)
            self.btn_fps.setDisabled(True)
            self.is_change = False
        else:
            self.btn_depth.setDisabled(False)
            self.btn_fps.setDisabled(False)
            self.is_change = True

    def set_config(self) -> None:
        _config_sidebar["depth_mode"] = self.btn_depth.currentIndex()+1
        all_signals.config_viewer.emit(_config_sidebar)


class IRCameraPanel(QFrame):
    def __init__(self) -> None:
        super().__init__()
        self.setObjectName("IRCameraPanel")
        self.setFixedHeight(100)
        self.setStyleSheet("""
            QFrame#IRCameraPanel {
                border-color: gray; border-width: 2px; border-radius: 5px;
            }
        """)

        layout_grid = QGridLayout()
        layout_title = QHBoxLayout()
        self.is_change = True

        layout_grid.addWidget(QLabel("<b>IR Camera Option<b>"), 0, 0)
        self.btn_switch = ToggleButton()
        self.btn_switch.clicked.connect(self._toggle)
        layout_title.addWidget(QLabel(""))
        layout_title.addWidget(self.btn_switch)
        layout_grid.addLayout(layout_title, 0, 1)       

        self.setLayout(layout_grid)

    def _toggle(self) -> None:
        if self.is_change:
            self.is_change = False
        else:
            self.is_change = True


class AudioPanel(QFrame):
    def __init__(self) -> None:
        super().__init__()
        self.setObjectName("AudioPanel")
        # self.setFixedSize(280, 200)
        self.setStyleSheet("""
            QFrame#AudioPanel {
                border-color: gray; border-width: 2px; border-radius: 5px;
            }
        """)
        
        layout_grid = QGridLayout()
        layout_title = QHBoxLayout()
        self.is_change = True

        layout_grid.addWidget(QLabel("<b>Audio Option<b>"), 0, 0)
        self.btn_switch = ToggleButton()
        self.btn_switch.clicked.connect(self._toggle)
        layout_title.addWidget(QLabel(""))
        layout_title.addWidget(self.btn_switch)
        layout_grid.addLayout(layout_title, 0, 1) 
        
        layout_grid.addWidget(QLabel("Samplerate"), 1, 0)
        self.btn_samplerate = ComboBox(
            ["22050", "44100"], 1
        )
        layout_grid.addWidget(self.btn_samplerate, 1, 1)
        self.btn_samplerate.currentIndexChanged.connect(self.set_config)

        layout_grid.addWidget(QLabel("Audio Channels"), 2, 0)
        self.btn_channel = ComboBox(
            ["1", "2", "3", "4", "5", "6", "7"], 1
        ) 
        layout_grid.addWidget(self.btn_channel, 2, 1)
        self.btn_channel.currentIndexChanged.connect(self.set_config)

        layout_grid.addWidget(QLabel("Subtype"), 3, 0)
        self.btn_subtype = ComboBox(
            ["PCM_S8", "PCM_16", "PCM_24"], 2
        )
        layout_grid.addWidget(self.btn_subtype, 3, 1)
        self.btn_subtype.currentIndexChanged.connect(self.set_config)

        self.set_config()
        self.setLayout(layout_grid)
        
    def _toggle(self) -> None:
        if self.is_change:
            self.btn_samplerate.setDisabled(True)
            self.btn_channel.setDisabled(True)
            self.btn_subtype.setDisabled(True)
            self.is_change = False
        else:
            self.btn_samplerate.setDisabled(False)
            self.btn_channel.setDisabled(False)
            self.btn_subtype.setDisabled(False)
            self.is_change = True

    def set_config(self) -> None:
        audio = {
            "samplerate": self.btn_samplerate.currentText(),
            "channel": self.btn_channel.currentText(),
            "subtype": self.btn_subtype.currentText()
        }
        _config_sidebar["audio"] = audio
        all_signals.config_viewer.emit(_config_sidebar)


class ToggleButton(QPushButton):
    def __init__(self, parent = None) -> None:
        super().__init__(parent)
        self.bg_color = QColor(0, 188, 248)

        self.setCheckable(True)
        self.setMinimumWidth(55)  # 55
        self.setMinimumHeight(22)  # 22

    def paintEvent(self, event) -> None:
        self.bg_color = QColor(255, 40, 40) if self.isChecked() else QColor(0, 188, 248)

        radius = 7
        width = 2*radius+2
        center = self.rect().center()

        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.translate(center)
        painter.setBrush(QColor(0, 0, 0))

        pen = QPen(Qt.black)
        pen.setWidth(2)
        pen.setStyle(Qt.PenStyle.MPenStyle)
        painter.setPen(pen)

        painter.setBrush(QColor(63, 64,66))
        painter.drawRoundedRect(QRect(-width-1, -radius-1, 2*width+2, 2*radius+2), 3, 3)
        
        painter.setBrush(self.bg_color)
        sw_rect = QRect(-width+2, -radius+1, 2*radius-2, 2*radius-2)
        if not self.isChecked():
            sw_rect.moveLeft(width-radius*2)
        painter.drawRoundedRect(sw_rect, 3, 3)


_config_sidebar = {
    "color": {
        "color_resolution": 1,
        "color_format": 0,
        "camera_fps": 2
    },
    "color_option": {
        "K4A_COLOR_CONTROL_EXPOSURE_TIME_ABSOLUTE": 33300,
        "K4A_COLOR_CONTROL_WHITEBALANCE": 4500,
        "K4A_COLOR_CONTROL_CONTRAST": 5,
        "K4A_COLOR_CONTROL_SATURATION": 32,
        "K4A_COLOR_CONTROL_SHARPNESS": 2,
        "K4A_COLOR_CONTROL_BRIGHTNESS": 128,
        "K4A_COLOR_CONTROL_GAIN": 128,
        "K4A_COLOR_CONTROL_BACKLIGHT_COMPENSATION": 1,
        "K4A_COLOR_CONTROL_POWERLINE_FREQUENCY": 2,
    },
    "depth_mode": 2,
}


import numpy as np
from pykinect_azure.k4abt._k4abtTypes import K4ABT_JOINT_NAMES


class Joint:
    def __init__(self, joint_handle, id):
        if joint_handle:
            self._handle = joint_handle
            self.position = joint_handle.position.xyz
            self.orientation = joint_handle.orientation.wxyz
            self.confidence_level = joint_handle.confidence_level
            self.id = id
            self.name = self.get_name()

    def __del__(self):
        self.destroy()

    def numpy(self):
        return np.array(
            [
                self.position.x,
                self.position.y,
                self.position.z,
                self.orientation.w,
                self.orientation.x,
                self.orientation.y,
                self.orientation.z,
                self.confidence_level,
            ]
        )

    def is_valid(self):
        return self._handle

    def handle(self):
        return self._handle

    def destroy(self):
        if self.is_valid():
            self._handle = None

    def get_name(self):
        return K4ABT_JOINT_NAMES[self.id]

    def __str__(self):
        """Print the current settings and a short explanation"""
        message = (
            f"{self.name} Joint info: \n"
            f"\tposition: [{self.position.x},{self.position.y},{self.position.z}]\n"
            f"\torientation: [{self.orientation.w},{self.orientation.x},{self.orientation.y},{self.orientation.z}]\n"
            f"\tconfidence: {self.confidence_level} \n\n"
        )
        return message



from PySide6.QtCore import Slot
from PySide6.QtWidgets import (
    QStackedLayout, QFrame
)

from .viewer_sensors import SensorViewer 
from .viewer_playback import PlaybackViewer
from .viewer_solution import ViewerSolution
from .asidebar import Asidebar


class StackedViewer(QFrame):
    def __init__(self) -> None:
        super().__init__()

        self.setMaximumHeight(1080)
        self.setMaximumWidth(1300)
        self.main_layout = QStackedLayout()
        self.main_viewer = SensorViewer()
        self.main_explorer = PlaybackViewer()
        self.main_solution = ViewerSolution()
        
        self.main_layout.addWidget(self.main_viewer)
        self.main_layout.addWidget(self.main_explorer)
        self.main_layout.addWidget(self.main_solution)

        # 현재 index
        self.main_layout.setCurrentIndex(0)
        self.setLayout(self.main_layout)


    @Slot(str)
    def setCurrentWidget(self, value):
        if value == "explorer":
            self.main_layout.setCurrentWidget(self.main_explorer)
        elif value == "solution":
            self.main_layout.setCurrentWidget(self.main_solution)
        else:
            self.main_layout.setCurrentWidget(self.main_viewer)


import ctypes
import sys
import traceback
from ._k4atypes import *

k4a_dll = None


def setup_library(module_k4a_path):
    global k4a_dll

    try:
        k4a_dll = ctypes.CDLL(module_k4a_path)

    except Exception as e:
        print("Failed to load library", e)
        sys.exit(1)

# ?
def k4a_device_get_installed_count():
    # K4A_EXPORT uint32_t k4a_device_get_installed_count(void);

    return k4a_dll.k4a_device_get_installed_count()

def k4a_device_open(device_id, device_handle):
    # K4A_EXPORT k4a_result_t k4a_device_open(uint32_t index, k4a_device_t *device_handle);

    _k4a_device_open = k4a_dll.k4a_device_open
    _k4a_device_open.restype = ctypes.c_int
    _k4a_device_open.argtypes = (ctypes.c_uint32, ctypes.POINTER(k4a_device_t))

    return _k4a_device_open(device_id, device_handle)


def k4a_device_close(device_handle):
    # K4A_EXPORT void k4a_device_close(k4a_device_t device_handle);

    _k4a_device_close = k4a_dll.k4a_device_close
    _k4a_device_close.restype = None
    _k4a_device_close.argtypes = (k4a_device_t,)
    _k4a_device_close(device_handle)


def k4a_device_get_capture(device_handle, capture_handle, timeout):
    """
    K4A_EXPORT k4a_wait_result_t k4a_device_get_capture(k4a_device_t device_handle,
                                                                                                            k4a_capture_t *capture_handle,
                                                                                                            int32_t timeout_in_ms);
    """

    _k4a_device_get_capture = k4a_dll.k4a_device_get_capture
    _k4a_device_get_capture.restype = ctypes.c_int
    _k4a_device_get_capture.argtypes = (
        k4a_device_t,
        ctypes.POINTER(k4a_capture_t),
        ctypes.c_int32,
    )

    return _k4a_device_get_capture(device_handle, capture_handle, timeout)


def k4a_device_get_imu_sample(device_handle, imu_sample_handle, timeout):
    """
    K4A_EXPORT k4a_wait_result_t k4a_device_get_imu_sample(
        k4a_device_t device_handle,
        k4a_imu_sample_t *imu_sample,
        int32_t timeout_in_ms
    );
    """

    _k4a_device_get_imu_sample = k4a_dll.k4a_device_get_imu_sample
    _k4a_device_get_imu_sample.restype = ctypes.c_int
    _k4a_device_get_imu_sample.argtypes = (
        k4a_device_t,
        ctypes.POINTER(k4a_imu_sample_t),
        ctypes.c_int32,
    )

    return _k4a_device_get_imu_sample(device_handle, imu_sample_handle, timeout)


def k4a_capture_create(capture_handle):
    # K4A_EXPORT k4a_result_t k4a_capture_create(k4a_capture_t *capture_handle);

    _k4a_capture_create = k4a_dll.k4a_capture_create
    _k4a_capture_create.restype = k4a_result_t
    _k4a_capture_create.argtypes = (ctypes.POINTER(k4a_capture_t),)

    return _k4a_capture_create(capture_handle)


def k4a_capture_release(capture_handle):
    # K4A_EXPORT void k4a_capture_release(k4a_capture_t capture_handle);

    _k4a_capture_release = k4a_dll.k4a_capture_release
    _k4a_capture_release.restype = None
    _k4a_capture_release.argtypes = (k4a_capture_t,)

    _k4a_capture_release(capture_handle)


def k4a_capture_reference(capture_handle):
    # K4A_EXPORT void k4a_capture_reference(k4a_capture_t capture_handle);

    _k4a_capture_reference = k4a_dll.k4a_capture_reference
    _k4a_capture_reference.restype = None
    _k4a_capture_reference.argtypes = (k4a_capture_t,)

    _k4a_capture_reference(capture_handle)


def k4a_capture_get_color_image(capture_handle):
    # K4A_EXPORT k4a_image_t k4a_capture_get_color_image(k4a_capture_t capture_handle)

    _k4a_capture_get_color_image = k4a_dll.k4a_capture_get_color_image
    _k4a_capture_get_color_image.restype = k4a_image_t
    _k4a_capture_get_color_image.argtypes = (k4a_capture_t,)

    return _k4a_capture_get_color_image(capture_handle)


def k4a_capture_get_depth_image(capture_handle):
    # K4A_EXPORT k4a_image_t k4a_capture_get_depth_image(k4a_capture_t capture_handle);

    _k4a_capture_get_depth_image = k4a_dll.k4a_capture_get_depth_image
    _k4a_capture_get_depth_image.restype = k4a_image_t
    _k4a_capture_get_depth_image.argtypes = (k4a_capture_t,)

    return _k4a_capture_get_depth_image(capture_handle)


def k4a_capture_get_ir_image(capture_handle):
    # K4A_EXPORT k4a_image_t k4a_capture_get_ir_image(k4a_capture_t capture_handle);

    _k4a_capture_get_ir_image = k4a_dll.k4a_capture_get_ir_image
    _k4a_capture_get_ir_image.restype = k4a_image_t
    _k4a_capture_get_ir_image.argtypes = (k4a_capture_t,)

    return _k4a_capture_get_ir_image(capture_handle)


def k4a_capture_set_color_image(capture_handle, image_handle):
    # K4A_EXPORT void k4a_capture_set_color_image(k4a_capture_t capture_handle, k4a_image_t image_handle);

    _k4a_capture_set_color_image = k4a_dll.k4a_capture_set_color_image
    _k4a_capture_set_color_image.restype = None
    _k4a_capture_set_color_image.argtypes = (
        k4a_capture_t,
        k4a_image_t,
    )

    _k4a_capture_set_color_image(capture_handle, image_handle)


def k4a_capture_set_depth_image(capture_handle, image_handle):
    # K4A_EXPORT void k4a_capture_set_depth_image(k4a_capture_t capture_handle, k4a_image_t image_handle);

    _k4a_capture_set_depth_image = k4a_dll.k4a_capture_set_depth_image
    _k4a_capture_set_depth_image.restype = None
    _k4a_capture_set_depth_image.argtypes = (
        k4a_capture_t,
        k4a_image_t,
    )

    _k4a_capture_set_depth_image(capture_handle, image_handle)


def k4a_capture_set_ir_image(capture_handle, image_handle):
    # K4A_EXPORT void k4a_capture_set_ir_image(k4a_capture_t capture_handle, k4a_image_t image_handle);

    _k4a_capture_set_ir_image = k4a_dll.k4a_capture_set_ir_image
    _k4a_capture_set_ir_image.restype = None
    _k4a_capture_set_ir_image.argtypes = (
        k4a_capture_t,
        k4a_image_t,
    )

    _k4a_capture_set_ir_image(capture_handle, image_handle)


def k4a_capture_set_temperature_c(capture_handle, temperature):
    # K4A_EXPORT void k4a_capture_set_temperature_c(k4a_capture_t capture_handle, float temperature_c);

    _k4a_capture_set_temperature_c = k4a_dll.k4a_capture_set_temperature_c
    _k4a_capture_set_temperature_c.restype = None
    _k4a_capture_set_temperature_c.argtypes = (
        k4a_capture_t,
        ctypes.c_float,
    )

    _k4a_capture_set_temperature_c(capture_handle, temperature)


def k4a_capture_get_temperature_c(capture_handle):
    # K4A_EXPORT float k4a_capture_get_temperature_c(k4a_capture_t capture_handle);

    _k4a_capture_get_temperature_c = k4a_dll.k4a_capture_get_temperature_c
    _k4a_capture_get_temperature_c.restype = ctypes.c_float
    _k4a_capture_get_temperature_c.argtypes = (k4a_capture_t,)

    return _k4a_capture_get_temperature_c(capture_handle)


def k4a_image_create(image_format, width, height, stride, image_handle):
    """
    K4A_EXPORT k4a_result_t k4a_image_create(k4a_image_format_t format,
                                                                                            int width_pixels,
                                                                                            int height_pixels,
                                                                                            int stride_bytes,
                                                                                            k4a_image_t *image_handle);
    """
    _k4a_image_create = k4a_dll.k4a_image_create
    _k4a_image_create.restype = k4a_result_t
    _k4a_image_create.argtypes = (
        k4a_image_format_t,
        ctypes.c_int,
        ctypes.c_int,
        ctypes.c_int,
        ctypes.POINTER(k4a_image_t),
    )

    return _k4a_image_create(image_format, width, height, stride, image_handle)


def k4a_image_create_from_buffer(
    image_format,
    width,
    height,
    stride,
    buffer,
    buffer_size,
    buffer_release_cb,
    buffer_release_cb_context,
    image_handle,
):
    """
    K4A_EXPORT k4a_result_t k4a_image_create_from_buffer(k4a_image_format_t format,
                                                                                                                    int width_pixels,
                                                                                                                    int height_pixels,
                                                                                                                    int stride_bytes,
                                                                                                                    uint8_t *buffer,
                                                                                                                    size_t buffer_size,
                                                                                                                    k4a_memory_destroy_cb_t *buffer_release_cb,
                                                                                                                    void *buffer_release_cb_context,
                                                                                                                    k4a_image_t *image_handle);
    """
    _k4a_image_create_from_buffer = k4a_dll.k4a_image_create_from_buffer
    _k4a_image_create_from_buffer.restype = k4a_result_t
    _k4a_image_create_from_buffer.argtypes = (
        k4a_image_format_t,
        ctypes.c_int,
        ctypes.c_int,
        ctypes.c_int,
        ctypes.POINTER(ctypes.c_uint8),
        ctypes.c_size_t,
        ctypes.c_void_p,
        ctypes.c_void_p,
        ctypes.POINTER(k4a_image_t),
    )

    return _k4a_image_create_from_buffer(
        image_format,
        width,
        height,
        stride,
        buffer,
        buffer_size,
        buffer_release_cb,
        buffer_release_cb_context,
        image_handle,
    )


def k4a_image_get_buffer(image_handle):
    # K4A_EXPORT uint8_t *k4a_image_get_buffer(k4a_image_t image_handle);

    _k4a_image_get_buffer = k4a_dll.k4a_image_get_buffer
    _k4a_image_get_buffer.restype = ctypes.POINTER(ctypes.c_uint8)
    _k4a_image_get_buffer.argtypes = (k4a_image_t,)

    return _k4a_image_get_buffer(image_handle)


def k4a_image_get_size(image_handle):
    # K4A_EXPORT size_t k4a_image_get_size(k4a_image_t image_handle);

    _k4a_image_get_size = k4a_dll.k4a_image_get_size
    _k4a_image_get_size.restype = ctypes.c_size_t
    _k4a_image_get_size.argtypes = (k4a_image_t,)

    return _k4a_image_get_size(image_handle)


def k4a_image_get_format(image_handle):
    # K4A_EXPORT k4a_image_format_t k4a_image_get_format(k4a_image_t image_handle);

    _k4a_image_get_format = k4a_dll.k4a_image_get_format
    _k4a_image_get_format.restype = k4a_image_format_t
    _k4a_image_get_format.argtypes = (k4a_image_t,)

    return _k4a_image_get_format(image_handle)


def k4a_image_get_width_pixels(image_handle):
    # K4A_EXPORT int k4a_image_get_width_pixels(k4a_image_t image_handle);

    _k4a_image_get_width_pixels = k4a_dll.k4a_image_get_width_pixels
    _k4a_image_get_width_pixels.restype = ctypes.c_int
    _k4a_image_get_width_pixels.argtypes = (k4a_image_t,)

    return _k4a_image_get_width_pixels(image_handle)


def k4a_image_get_height_pixels(image_handle):
    # K4A_EXPORT int k4a_image_get_height_pixels(k4a_image_t image_handle);

    _k4a_image_get_height_pixels = k4a_dll.k4a_image_get_height_pixels
    _k4a_image_get_height_pixels.restype = ctypes.c_int
    _k4a_image_get_height_pixels.argtypes = (k4a_image_t,)

    return _k4a_image_get_height_pixels(image_handle)


def k4a_image_get_stride_bytes(image_handle):
    # K4A_EXPORT int k4a_image_get_stride_bytes(k4a_image_t image_handle);

    _k4a_image_get_stride_bytes = k4a_dll.k4a_image_get_stride_bytes
    _k4a_image_get_stride_bytes.restype = ctypes.c_int
    _k4a_image_get_stride_bytes.argtypes = (k4a_image_t,)

    return _k4a_image_get_stride_bytes(image_handle)


def k4a_image_get_timestamp_usec(image_handle):
    # K4A_DEPRECATED_EXPORT uint64_t k4a_image_get_timestamp_usec(k4a_image_t image_handle);

    _k4a_image_get_timestamp_usec = k4a_dll.k4a_image_get_timestamp_usec
    _k4a_image_get_timestamp_usec.restype = ctypes.c_uint64
    _k4a_image_get_timestamp_usec.argtypes = (k4a_image_t,)

    return _k4a_image_get_timestamp_usec(image_handle)


def k4a_image_get_device_timestamp_usec(image_handle):
    # K4A_EXPORT uint64_t k4a_image_get_device_timestamp_usec(k4a_image_t image_handle);

    _k4a_image_get_device_timestamp_usec = k4a_dll.k4a_image_get_device_timestamp_usec
    _k4a_image_get_device_timestamp_usec.restype = ctypes.c_uint64
    _k4a_image_get_device_timestamp_usec.argtypes = (k4a_image_t,)

    return _k4a_image_get_device_timestamp_usec(image_handle)


def k4a_image_get_system_timestamp_nsec(image_handle):
    # K4A_EXPORT uint64_t k4a_image_get_system_timestamp_nsec(k4a_image_t image_handle);

    _k4a_image_get_system_timestamp_nsec = k4a_dll.k4a_image_get_system_timestamp_nsec
    _k4a_image_get_system_timestamp_nsec.restype = ctypes.c_uint64
    _k4a_image_get_system_timestamp_nsec.argtypes = (k4a_image_t,)

    return _k4a_image_get_system_timestamp_nsec(image_handle)


def k4a_image_get_exposure_usec(image_handle):
    # K4A_EXPORT uint64_t k4a_image_get_exposure_usec(k4a_image_t image_handle);

    _k4a_image_get_exposure_usec = k4a_dll.k4a_image_get_exposure_usec
    _k4a_image_get_exposure_usec.restype = ctypes.c_uint64
    _k4a_image_get_exposure_usec.argtypes = (k4a_image_t,)

    return _k4a_image_get_exposure_usec(image_handle)


def k4a_image_get_white_balance(image_handle):
    # K4A_EXPORT uint32_t k4a_image_get_white_balance(k4a_image_t image_handle);

    _k4a_image_get_white_balance = k4a_dll.k4a_image_get_white_balance
    _k4a_image_get_white_balance.restype = ctypes.c_uint32
    _k4a_image_get_white_balance.argtypes = (k4a_image_t,)

    return _k4a_image_get_white_balance(image_handle)


def k4a_image_get_iso_speed(image_handle):
    # K4A_EXPORT uint32_t k4a_image_get_iso_speed(k4a_image_t image_handle);

    _k4a_image_get_iso_speed = k4a_dll.k4a_image_get_iso_speed
    _k4a_image_get_iso_speed.restype = ctypes.c_uint32
    _k4a_image_get_iso_speed.argtypes = (k4a_image_t,)

    return _k4a_image_get_iso_speed(image_handle)


def k4a_image_set_device_timestamp_usec(image_handle, timestamp_usec):
    # K4A_EXPORT void k4a_image_set_device_timestamp_usec(k4a_image_t image_handle, uint64_t timestamp_usec);

    _k4a_image_set_device_timestamp_usec = k4a_dll.k4a_image_set_device_timestamp_usec
    _k4a_image_set_device_timestamp_usec.restype = None
    _k4a_image_set_device_timestamp_usec.argtypes = (
        k4a_image_t,
        ctypes.c_uint64,
    )

    _k4a_image_set_device_timestamp_usec(image_handle, timestamp_usec)


def k4a_image_set_timestamp_usec(image_handle, timestamp_usec):
    # K4A_DEPRECATED_EXPORT void k4a_image_set_timestamp_usec(k4a_image_t image_handle, uint64_t timestamp_usec);

    _k4a_image_set_timestamp_usec = k4a_dll.k4a_image_set_timestamp_usec
    _k4a_image_set_timestamp_usec.restype = None
    _k4a_image_set_timestamp_usec.argtypes = (
        k4a_image_t,
        ctypes.c_uint64,
    )

    _k4a_image_set_timestamp_usec(image_handle, timestamp_usec)


def k4a_image_set_system_timestamp_nsec(image_handle, timestamp_nsec):
    # K4A_EXPORT void k4a_image_set_system_timestamp_nsec(k4a_image_t image_handle, uint64_t timestamp_nsec);

    _k4a_image_set_system_timestamp_nsec = k4a_dll.k4a_image_set_system_timestamp_nsec
    _k4a_image_set_system_timestamp_nsec.restype = None
    _k4a_image_set_system_timestamp_nsec.argtypes = (
        k4a_image_t,
        ctypes.c_uint64,
    )

    _k4a_image_set_system_timestamp_nsec(image_handle, timestamp_nsec)


def k4a_image_set_exposure_usec(image_handle, exposure_usec):
    # K4A_EXPORT void k4a_image_set_exposure_usec(k4a_image_t image_handle, uint64_t exposure_usec);

    _k4a_image_set_exposure_usec = k4a_dll.k4a_image_set_exposure_usec
    _k4a_image_set_exposure_usec.restype = None
    _k4a_image_set_exposure_usec.argtypes = (
        k4a_image_t,
        ctypes.c_uint64,
    )

    _k4a_image_set_exposure_usec(image_handle, exposure_usec)


def k4a_image_set_exposure_time_usec(image_handle, exposure_usec):
    # K4A_DEPRECATED_EXPORT void k4a_image_set_exposure_time_usec(k4a_image_t image_handle, uint64_t exposure_usec);

    _k4a_image_set_exposure_time_usec = k4a_dll.k4a_image_set_exposure_time_usec
    _k4a_image_set_exposure_time_usec.restype = None
    _k4a_image_set_exposure_time_usec.argtypes = (
        k4a_image_t,
        ctypes.c_uint64,
    )

    _k4a_image_set_exposure_time_usec(image_handle, exposure_usec)


def k4a_image_set_white_balance(image_handle, white_balance):
    # K4A_EXPORT void k4a_image_set_white_balance(k4a_image_t image_handle, uint32_t white_balance);

    _k4a_image_set_white_balance = k4a_dll.k4a_image_set_white_balance
    _k4a_image_set_white_balance.restype = None
    _k4a_image_set_white_balance.argtypes = (
        k4a_image_t,
        ctypes.c_uint32,
    )

    _k4a_image_set_white_balance(image_handle, white_balance)


def k4a_image_set_iso_speed(image_handle, iso_speed):
    # K4A_EXPORT void k4a_image_set_iso_speed(k4a_image_t image_handle, uint32_t iso_speed);

    _k4a_image_set_iso_speed = k4a_dll.k4a_image_set_iso_speed
    _k4a_image_set_iso_speed.restype = None
    _k4a_image_set_iso_speed.argtypes = (
        k4a_image_t,
        ctypes.c_uint32,
    )

    _k4a_image_set_iso_speed(image_handle, iso_speed)


def k4a_image_reference(image_handle):
    # K4A_EXPORT void k4a_image_reference(k4a_image_t image_handle);

    _k4a_image_reference = k4a_dll.k4a_image_reference
    _k4a_image_reference.restype = None
    _k4a_image_reference.argtypes = (k4a_image_t,)

    _k4a_image_reference(image_handle)


def k4a_image_release(image_handle):
    # K4A_EXPORT void k4a_image_release(k4a_image_t image_handle);

    _k4a_image_release = k4a_dll.k4a_image_release
    _k4a_image_release.restype = None
    _k4a_image_release.argtypes = (k4a_image_t,)

    _k4a_image_release(image_handle)


def k4a_device_start_cameras(device_handle, config):
    # K4A_EXPORT k4a_result_t k4a_device_start_cameras(k4a_device_t device_handle, const k4a_device_configuration_t *config);

    _k4a_device_start_cameras = k4a_dll.k4a_device_start_cameras
    _k4a_device_start_cameras.restype = k4a_result_t
    _k4a_device_start_cameras.argtypes = (
        k4a_device_t,
        ctypes.POINTER(k4a_device_configuration_t),
    )

    return _k4a_device_start_cameras(device_handle, config)


def k4a_device_stop_cameras(device_handle):
    # K4A_EXPORT void k4a_device_stop_cameras(k4a_device_t device_handle);

    _k4a_device_stop_cameras = k4a_dll.k4a_device_stop_cameras
    _k4a_device_stop_cameras.restype = None
    _k4a_device_stop_cameras.argtypes = (k4a_device_t,)

    _k4a_device_stop_cameras(device_handle)


def k4a_device_start_imu(device_handle):
    # K4A_EXPORT k4a_result_t k4a_device_start_imu(k4a_device_t device_handle);

    _k4a_device_start_imu = k4a_dll.k4a_device_start_imu
    _k4a_device_start_imu.restype = k4a_result_t
    _k4a_device_start_imu.argtypes = (k4a_device_t,)

    return _k4a_device_start_imu(device_handle)


def k4a_device_stop_imu(device_handle):
    # K4A_EXPORT void k4a_device_stop_imu(k4a_device_t device_handle);

    _k4a_device_stop_imu = k4a_dll.k4a_device_stop_imu
    _k4a_device_stop_imu.restype = None
    _k4a_device_stop_imu.argtypes = (k4a_device_t,)

    _k4a_device_stop_imu(device_handle)


def k4a_device_get_serialnum(device_handle, serial_number, serial_number_size):
    """
    K4A_EXPORT k4a_buffer_result_t k4a_device_get_serialnum(
        k4a_device_t device_handle,
        char *serial_number,
        size_t *serial_number_size
    );
    """

    _k4a_device_get_serialnum = k4a_dll.k4a_device_get_serialnum
    _k4a_device_get_serialnum.restype = k4a_buffer_result_t
    _k4a_device_get_serialnum.argtypes = (
        k4a_device_t,
        ctypes.c_char_p,
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_device_get_serialnum(device_handle, serial_number, serial_number_size)


def k4a_device_get_version(device_handle, hardware_version):
    # K4A_EXPORT k4a_result_t k4a_device_get_version(k4a_device_t device_handle, k4a_hardware_version_t *version);

    _k4a_device_get_version = k4a_dll.k4a_device_get_version
    _k4a_device_get_version.restype = k4a_result_t
    _k4a_device_get_version.argtypes = (
        k4a_device_t,
        ctypes.POINTER(k4a_hardware_version_t),
    )

    return _k4a_device_get_version(device_handle, hardware_version)


def k4a_device_get_color_control_capabilities(
    device_handle,
    command,
    supports_auto,
    min_value,
    max_value,
    step_value,
    default_value,
    default_mode,
):
    """
    K4A_EXPORT k4a_result_t k4a_device_get_color_control_capabilities(
        k4a_device_t device_handle,
        k4a_color_control_command_t command,
        bool *supports_auto,
        int32_t *min_value,
        int32_t *max_value,
        int32_t *step_value,
        int32_t *default_value,
        k4a_color_control_mode_t *default_mode
    );
    """

    _k4a_device_get_color_control_capabilities = (
        k4a_dll.k4a_device_get_color_control_capabilities
    )
    _k4a_device_get_color_control_capabilities.restype = k4a_result_t
    _k4a_device_get_color_control_capabilities.argtypes = (
        k4a_device_t,
        k4a_color_control_command_t,
        ctypes.POINTER(ctypes.c_bool),
        ctypes.POINTER(ctypes.c_int32),
        ctypes.POINTER(ctypes.c_int32),
        ctypes.POINTER(ctypes.c_int32),
        ctypes.POINTER(ctypes.c_int32),
        ctypes.POINTER(k4a_color_control_mode_t),
    )

    return _k4a_device_get_color_control_capabilities(
        device_handle,
        command,
        supports_auto,
        min_value,
        max_value,
        step_value,
        default_value,
        default_mode,
    )


def k4a_device_get_color_control(device_handle, command, mode, value):
    """
    K4A_EXPORT k4a_result_t k4a_device_get_color_control(
        k4a_device_t device_handle,
        k4a_color_control_command_t command,
        k4a_color_control_mode_t *mode,
        int32_t *value
    );
    """

    _k4a_device_get_color_control = k4a_dll.k4a_device_get_color_control
    _k4a_device_get_color_control.restype = k4a_result_t
    _k4a_device_get_color_control.argtypes = (
        k4a_device_t,
        k4a_color_control_command_t,
        ctypes.POINTER(k4a_color_control_mode_t),
        ctypes.POINTER(ctypes.c_int32),
    )

    return _k4a_device_get_color_control(device_handle, command, mode, value)


def k4a_device_set_color_control(device_handle, command, mode, value):
    """
    K4A_EXPORT k4a_result_t k4a_device_set_color_control(
        k4a_device_t device_handle,
        k4a_color_control_command_t command,
        k4a_color_control_mode_t mode,
        int32_t value
    );
    """

    _k4a_device_set_color_control = k4a_dll.k4a_device_set_color_control
    _k4a_device_set_color_control.restype = k4a_result_t
    _k4a_device_set_color_control.argtypes = (
        k4a_device_t,
        k4a_color_control_command_t,
        k4a_color_control_mode_t,
        ctypes.c_int32,
    )

    return _k4a_device_set_color_control(device_handle, command, mode, value)


def k4a_device_get_raw_calibration(device_handle, data, data_size):
    """
    K4A_EXPORT k4a_buffer_result_t k4a_device_get_raw_calibration(
        k4a_device_t device_handle,
        uint8_t *data,
        size_t *data_size
    );
    """

    _k4a_device_get_raw_calibration = k4a_dll.k4a_device_get_raw_calibration
    _k4a_device_get_raw_calibration.restype = k4a_buffer_result_t
    _k4a_device_get_raw_calibration.argtypes = (
        k4a_device_t,
        ctypes.POINTER(ctypes.c_uint8),
        ctypes.POINTER(ctypes.c_size_t),
    )

    return _k4a_device_get_raw_calibration(device_handle, data, data_size)


def k4a_device_get_calibration(
    device_handle, depth_mode, color_resolution, calibration
):
    """
    K4A_EXPORT k4a_result_t k4a_device_get_calibration(
        k4a_device_t device_handle,
        const k4a_depth_mode_t depth_mode,
        const k4a_color_resolution_t color_resolution,
        k4a_calibration_t *calibration
    );
    """

    _k4a_device_get_calibration = k4a_dll.k4a_device_get_calibration
    _k4a_device_get_calibration.restype = k4a_result_t
    _k4a_device_get_calibration.argtypes = (
        k4a_device_t,
        k4a_depth_mode_t,
        k4a_color_resolution_t,
        ctypes.POINTER(k4a_calibration_t),
    )

    return _k4a_device_get_calibration(
        device_handle, depth_mode, color_resolution, calibration
    )


def k4a_device_get_sync_jack(
    device_handle, sync_in_jack_connected, sync_out_jack_connected
):
    """
    K4A_EXPORT k4a_result_t k4a_device_get_sync_jack(
        k4a_device_t device_handle,
        bool *sync_in_jack_connected,
        bool *sync_out_jack_connected
    );
    """

    _k4a_device_get_sync_jack = k4a_dll.k4a_device_get_sync_jack
    _k4a_device_get_sync_jack.restype = k4a_result_t
    _k4a_device_get_sync_jack.argtypes = (
        k4a_device_t,
        ctypes.POINTER(ctypes.c_bool),
        ctypes.POINTER(ctypes.c_bool),
    )

    return _k4a_device_get_sync_jack(
        device_handle, sync_in_jack_connected, sync_out_jack_connected
    )


def k4a_calibration_get_from_raw(
    raw_calibration, raw_calibration_size, depth_mode, color_resolution, calibration
):
    """
    K4A_EXPORT k4a_result_t k4a_calibration_get_from_raw(
        char *raw_calibration,
        size_t raw_calibration_size,
        const k4a_depth_mode_t depth_mode,
        const k4a_color_resolution_t color_resolution,
        k4a_calibration_t *calibration
    );
    """

    _k4a_calibration_get_from_raw = k4a_dll.k4a_calibration_get_from_raw
    _k4a_calibration_get_from_raw.restype = k4a_result_t
    _k4a_calibration_get_from_raw.argtypes = (
        ctypes.POINTER(ctypes.c_char),
        ctypes.c_size_t,
        k4a_depth_mode_t,
        k4a_color_resolution_t,
        ctypes.POINTER(k4a_calibration_t),
    )

    return _k4a_calibration_get_from_raw(
        raw_calibration, raw_calibration_size, depth_mode, color_resolution, calibration
    )


def k4a_calibration_3d_to_3d(
    calibration, source_point3d_mm, source_camera, target_camera, target_point3d_mm
):
    """
    K4A_EXPORT k4a_result_t k4a_calibration_3d_to_3d(
        const k4a_calibration_t *calibration,
        const k4a_float3_t *source_point3d_mm,
        const k4a_calibration_type_t source_camera,
        const k4a_calibration_type_t target_camera,
        k4a_float3_t *target_point3d_mm
    );
    """

    _k4a_calibration_3d_to_3d = k4a_dll.k4a_calibration_3d_to_3d
    _k4a_calibration_3d_to_3d.restype = k4a_result_t
    _k4a_calibration_3d_to_3d.argtypes = (
        ctypes.POINTER(k4a_calibration_t),
        ctypes.POINTER(k4a_float3_t),
        k4a_calibration_type_t,
        k4a_calibration_type_t,
        ctypes.POINTER(k4a_float3_t),
    )

    return _k4a_calibration_3d_to_3d(
        calibration, source_point3d_mm, source_camera, target_camera, target_point3d_mm
    )


def k4a_calibration_2d_to_3d(
    calibration,
    source_point2d,
    source_depth_mm,
    source_camera,
    target_camera,
    target_point3d_mm,
    valid,
):
    """
    K4A_EXPORT k4a_result_t k4a_calibration_2d_to_3d(
        const k4a_calibration_t *calibration,
        const k4a_float2_t *source_point2d,
        const float source_depth_mm,
        const k4a_calibration_type_t source_camera,
        const k4a_calibration_type_t target_camera,
        k4a_float3_t *target_point3d_mm,
        int *valid
    );
    """

    _k4a_calibration_2d_to_3d = k4a_dll.k4a_calibration_2d_to_3d
    _k4a_calibration_2d_to_3d.restype = k4a_result_t
    _k4a_calibration_2d_to_3d.argtypes = (
        ctypes.POINTER(k4a_calibration_t),
        ctypes.POINTER(k4a_float2_t),
        ctypes.c_float,
        k4a_calibration_type_t,
        k4a_calibration_type_t,
        ctypes.POINTER(k4a_float3_t),
        ctypes.POINTER(ctypes.c_int),
    )

    return _k4a_calibration_2d_to_3d(
        calibration,
        source_point2d,
        source_depth_mm,
        source_camera,
        target_camera,
        target_point3d_mm,
        valid,
    )


def k4a_calibration_3d_to_2d(
    calibration, source_point3d_mm, source_camera, target_camera, target_point2d, valid
):
    """
    K4A_EXPORT k4a_result_t k4a_calibration_3d_to_2d(
        const k4a_calibration_t *calibration,
        const k4a_float3_t *source_point3d_mm,
        const k4a_calibration_type_t source_camera,
        const k4a_calibration_type_t target_camera,
        k4a_float2_t *target_point2d,
        int *valid
    );
    """

    _k4a_calibration_3d_to_2d = k4a_dll.k4a_calibration_3d_to_2d
    _k4a_calibration_3d_to_2d.restype = k4a_result_t
    _k4a_calibration_3d_to_2d.argtypes = (
        ctypes.POINTER(k4a_calibration_t),
        ctypes.POINTER(k4a_float3_t),
        k4a_calibration_type_t,
        k4a_calibration_type_t,
        ctypes.POINTER(k4a_float2_t),
        ctypes.POINTER(ctypes.c_int),
    )

    return _k4a_calibration_3d_to_2d(
        calibration,
        source_point3d_mm,
        source_camera,
        target_camera,
        target_point2d,
        valid,
    )


def k4a_calibration_2d_to_2d(
    calibration,
    source_point2d,
    source_depth_mm,
    source_camera,
    target_camera,
    target_point2d,
    valid,
):
    """
    K4A_EXPORT k4a_result_t k4a_calibration_2d_to_2d(
        const k4a_calibration_t *calibration,
        const k4a_float2_t *source_point2d,
        const float source_depth_mm,
        const k4a_calibration_type_t source_camera,
        const k4a_calibration_type_t target_camera,
        k4a_float2_t *target_point2d,
        int *valid
    );
    """

    _k4a_calibration_2d_to_2d = k4a_dll.k4a_calibration_2d_to_2d
    _k4a_calibration_2d_to_2d.restype = k4a_result_t
    _k4a_calibration_2d_to_2d.argtypes = (
        ctypes.POINTER(k4a_calibration_t),
        ctypes.POINTER(k4a_float2_t),
        ctypes.c_float,
        k4a_calibration_type_t,
        k4a_calibration_type_t,
        ctypes.POINTER(k4a_float2_t),
        ctypes.POINTER(ctypes.c_int),
    )

    return _k4a_calibration_2d_to_2d(
        calibration,
        source_point2d,
        source_depth_mm,
        source_camera,
        target_camera,
        target_point2d,
        valid,
    )


def k4a_calibration_color_2d_to_depth_2d(
    calibration, source_point2d, depth_image, target_point2d, valid
):
    """
    K4A_EXPORT k4a_result_t k4a_calibration_color_2d_to_depth_2d(
        const k4a_calibration_t *calibration,
        const k4a_float2_t *source_point2d,
        const k4a_image_t depth_image,
        k4a_float2_t *target_point2d,
        int *valid
    );
    """

    _k4a_calibration_color_2d_to_depth_2d = k4a_dll.k4a_calibration_color_2d_to_depth_2d
    _k4a_calibration_color_2d_to_depth_2d.restype = k4a_result_t
    _k4a_calibration_color_2d_to_depth_2d.argtypes = (
        ctypes.POINTER(k4a_calibration_t),
        ctypes.POINTER(k4a_float2_t),
        k4a_image_t,
        ctypes.POINTER(k4a_float2_t),
        ctypes.POINTER(ctypes.c_int),
    )

    return _k4a_calibration_color_2d_to_depth_2d(
        calibration, source_point2d, depth_image, target_point2d, valid
    )


def k4a_transformation_create(calibration):
    # K4A_EXPORT k4a_transformation_t k4a_transformation_create(const k4a_calibration_t *calibration);

    _k4a_transformation_create = k4a_dll.k4a_transformation_create
    _k4a_transformation_create.restype = k4a_transformation_t
    _k4a_transformation_create.argtypes = (ctypes.POINTER(k4a_calibration_t),)

    return _k4a_transformation_create(calibration)


def k4a_transformation_destroy(transformation_handle):
    # K4A_EXPORT void k4a_transformation_destroy(k4a_transformation_t transformation_handle);

    _k4a_transformation_destroy = k4a_dll.k4a_transformation_destroy
    _k4a_transformation_destroy.restype = None
    _k4a_transformation_destroy.argtypes = (k4a_transformation_t,)

    _k4a_transformation_destroy(transformation_handle)


def k4a_transformation_depth_image_to_color_camera(
    transformation_handle, depth_image, transformed_depth_image
):
    """
    K4A_EXPORT k4a_result_t k4a_transformation_depth_image_to_color_camera(
        k4a_transformation_t transformation_handle,
        const k4a_image_t depth_image,
        k4a_image_t transformed_depth_image
    );
    """

    _k4a_transformation_depth_image_to_color_camera = (
        k4a_dll.k4a_transformation_depth_image_to_color_camera
    )
    _k4a_transformation_depth_image_to_color_camera.restype = k4a_result_t
    _k4a_transformation_depth_image_to_color_camera.argtypes = (
        k4a_transformation_t,
        k4a_image_t,
        k4a_image_t,
    )

    _k4a_transformation_depth_image_to_color_camera(
        transformation_handle, depth_image, transformed_depth_image
    )


def k4a_transformation_depth_image_to_color_camera_custom(
    transformation_handle,
    depth_image,
    custom_image,
    transformed_depth_image,
    transformed_custom_image,
    interpolation_type,
    invalid_custom_value,
):
    """
    K4A_EXPORT k4a_result_t k4a_transformation_depth_image_to_color_camera_custom(
        k4a_transformation_t transformation_handle,
        const k4a_image_t depth_image,
        const k4a_image_t custom_image,
        k4a_image_t transformed_depth_image,
        k4a_image_t transformed_custom_image,
        k4a_transformation_interpolation_type_t interpolation_type,
        uint32_t invalid_custom_value
    );
    """

    _k4a_transformation_depth_image_to_color_camera_custom = (
        k4a_dll.k4a_transformation_depth_image_to_color_camera_custom
    )
    _k4a_transformation_depth_image_to_color_camera_custom.restype = k4a_result_t
    _k4a_transformation_depth_image_to_color_camera_custom.argtypes = (
        k4a_transformation_t,
        k4a_image_t,
        k4a_image_t,
        k4a_image_t,
        k4a_image_t,
        k4a_transformation_interpolation_type_t,
        ctypes.c_uint32,
    )

    return _k4a_transformation_depth_image_to_color_camera_custom(
        transformation_handle,
        depth_image,
        custom_image,
        transformed_depth_image,
        transformed_custom_image,
        interpolation_type,
        invalid_custom_value,
    )


def k4a_transformation_color_image_to_depth_camera(
    transformation_handle, depth_image, color_image, transformed_color_image
):
    """
    K4A_EXPORT k4a_result_t k4a_transformation_color_image_to_depth_camera(
        k4a_transformation_t transformation_handle,
        const k4a_image_t depth_image,
        const k4a_image_t color_image,
        k4a_image_t transformed_color_image
    );
    """

    _k4a_transformation_color_image_to_depth_camera = (
        k4a_dll.k4a_transformation_color_image_to_depth_camera
    )
    _k4a_transformation_color_image_to_depth_camera.restype = k4a_result_t
    _k4a_transformation_color_image_to_depth_camera.argtypes = (
        k4a_transformation_t,
        k4a_image_t,
        k4a_image_t,
        k4a_image_t,
    )

    return _k4a_transformation_color_image_to_depth_camera(
        transformation_handle, depth_image, color_image, transformed_color_image
    )


def k4a_transformation_depth_image_to_point_cloud(
    transformation_handle, depth_image, camera, xyz_image
):
    """
    K4A_EXPORT k4a_result_t k4a_transformation_depth_image_to_point_cloud(
        k4a_transformation_t transformation_handle,
        const k4a_image_t depth_image,
        const k4a_calibration_type_t camera,
        k4a_image_t xyz_image
    );
    """

    _k4a_transformation_depth_image_to_point_cloud = (
        k4a_dll.k4a_transformation_depth_image_to_point_cloud
    )
    _k4a_transformation_depth_image_to_point_cloud.restype = k4a_result_t
    _k4a_transformation_depth_image_to_point_cloud.argtypes = (
        k4a_transformation_t,
        k4a_image_t,
        k4a_calibration_type_t,
        k4a_image_t,
    )

    return _k4a_transformation_depth_image_to_point_cloud(
        transformation_handle, depth_image, camera, xyz_image
    )


def VERIFY(result, error):
    if result != K4A_RESULT_SUCCEEDED:
        print(error)
        # traceback.print_stack()
        sys.exit(1)



import numpy as np
import cv2

from . import _k4a


class Image:
    _handle = None
    buffer_pointer = None

    def __init__(self, image_handle=None):
        self._handle = image_handle
        # Get the pointer to the buffer containing the image data
        self.buffer_pointer = self.get_buffer() if self.is_valid() else None

    def __del__(self):
        self.reset()

    def is_valid(self):
        return self._handle or self.buffer_pointer is not None

    def handle(self):
        return self._handle

    def reset(self):
        if self.is_valid():
            _k4a.k4a_image_release(self._handle)
            self._handle = None

    @staticmethod
    def create(image_format, width_pixels, height_pixels, stride_bytes):
        handle = _k4a.k4a_image_t()
        _k4a.VERIFY(
            _k4a.k4a_image_create(
                image_format, width_pixels, height_pixels, stride_bytes, handle
            ),
            "Create image failed!",
        )

        return Image(handle)

    @property
    def width(self):
        return self.get_width_pixels()

    @property
    def height(self):
        return self.get_height_pixels()

    @property
    def stride(self):
        return self.get_stride_bytes()

    @property
    def format(self):
        return self.get_format()

    @property
    def size(self):
        return self.get_size()

    def get_buffer(self):
        if not self._handle:
            return None

        return _k4a.k4a_image_get_buffer(self._handle)

    def get_size(self):
        if not self.is_valid():
            return None

        return int(_k4a.k4a_image_get_size(self._handle))

    def get_format(self):
        if not self.is_valid():
            return None

        return int(_k4a.k4a_image_get_format(self._handle))

    def get_width_pixels(self):
        if not self.is_valid():
            return None

        return int(_k4a.k4a_image_get_width_pixels(self._handle))

    def get_height_pixels(self):
        if not self.is_valid():
            return None

        return int(_k4a.k4a_image_get_height_pixels(self._handle))

    def get_stride_bytes(self):
        return int(_k4a.k4a_image_get_stride_bytes(self._handle))

    def to_numpy(self):
        if not self.is_valid():
            return False, None

        # Get the size of the buffer
        image_size = self.get_size()
        image_width = self.get_width_pixels()
        image_height = self.get_height_pixels()

        # Get the image format
        image_format = self.get_format()

        # Read the data in the buffer
        buffer_array = np.ctypeslib.as_array(self.buffer_pointer, shape=(image_size,))

        # Parse buffer based on image formats
        if image_format == _k4a.K4A_IMAGE_FORMAT_COLOR_MJPG:
            return True, cv2.imdecode(
                np.frombuffer(buffer_array, dtype=np.uint8).copy(), -1
            )
        elif image_format == _k4a.K4A_IMAGE_FORMAT_COLOR_NV12:
            yuv_image = (
                np.frombuffer(buffer_array, dtype=np.uint8)
                .copy()
                .reshape(int(image_height * 1.5), image_width)
            )
            return True, cv2.cvtColor(yuv_image, cv2.COLOR_YUV2BGR_NV12)
        elif image_format == _k4a.K4A_IMAGE_FORMAT_COLOR_YUY2:
            yuv_image = (
                np.frombuffer(buffer_array, dtype=np.uint8)
                .copy()
                .reshape(image_height, image_width, 2)
            )
            return True, cv2.cvtColor(yuv_image, cv2.COLOR_YUV2BGR_YUY2)
        elif image_format == _k4a.K4A_IMAGE_FORMAT_COLOR_BGRA32:
            return True, np.frombuffer(buffer_array, dtype=np.uint8).copy().reshape(
                image_height, image_width, 4
            )
        elif image_format == _k4a.K4A_IMAGE_FORMAT_DEPTH16:
            return True, np.frombuffer(buffer_array, dtype="<u2").copy().reshape(
                image_height, image_width
            )  # little-endian 16 bits unsigned Depth data
        elif image_format == _k4a.K4A_IMAGE_FORMAT_IR16:
            return True, np.frombuffer(buffer_array, dtype="<u2").copy().reshape(
                image_height, image_width
            )  # little-endian 16 bits unsigned IR data. For more details see: https://microsoft.github.io/Azure-Kinect-Sensor-SDK/release/1.2.x/namespace_microsoft_1_1_azure_1_1_kinect_1_1_sensor_a7a3cb7a0a3073650bf17c2fef2bfbd1b.html
        elif image_format == _k4a.K4A_IMAGE_FORMAT_CUSTOM8:
            return True, np.frombuffer(buffer_array, dtype="<u1").copy().reshape(
                image_height, image_width
            )
        elif image_format == _k4a.K4A_IMAGE_FORMAT_CUSTOM16:
            return True, np.frombuffer(buffer_array, dtype="<u2").copy().reshape(
                image_height, image_width
            )
        elif image_format == _k4a.K4A_IMAGE_FORMAT_CUSTOM:
            return True, np.frombuffer(buffer_array, dtype="<i2").copy()



import numpy as np
import platform
import sys


def get_k4a_module_path():
    # Check if running in Jetson Nano or similar ARM chips
    if platform.machine().lower() == "aarch64":
        return r"/usr/lib/aarch64-linux-gnu/libk4a.so"

    # For non-Arm chips, first check if it is running linux
    if platform.system().lower() == "linux":
        return r"/usr/lib/x86_64-linux-gnu/libk4a.so"

    # In Windows check the architecture
    if platform.machine().lower() == "amd64":
        return "C:\\Program Files\\Azure Kinect SDK v1.4.1\\sdk\\windows-desktop\\amd64\\release\\bin\\k4a.dll"

    # Otherwise return the x86 Windows version
    return "C:\\Program Files\\Azure Kinect SDK v1.4.1\\sdk\\windows-desktop\\x86\\release\\bin\\k4a.dll"


def get_k4abt_module_path():
    # Check if running in Jetson Nano or similar ARM chips
    if platform.machine().lower() == "aarch64":
        print(
            "Kinect Body Tracking is not implemented yet in ARM. Check https://feedback.azure.com/forums/920053 for more info."
        )
        sys.exit(1)

    # For non-Arm chips, first check if it is running linux
    if platform.system().lower() == "linux":
        return "libk4abt.so"

    # Otherwise return the Windows version
    return "C:\\Program Files\\Azure Kinect Body Tracking SDK\\sdk\\windows-desktop\\amd64\\release\\bin\\k4abt.dll"


def get_k4arecord_module_path(modulePath):
    return modulePath.replace("k4a", "k4arecord")


def get_k4abt_lite_model_path():
    # Check if it is a Linux system
    if platform.system().lower() == "linux":
        return None

    # Return the Windows version
    return "C:/Program Files/Azure Kinect Body Tracking SDK/sdk/windows-desktop/amd64/release/bin/dnn_model_2_0_lite_op11.onnx".encode(
        "utf-8"
    )


def getdict(struct):
    result = {}
    for field, _ in struct._fields_:
        value = getattr(struct, field)
        # if the type is not a primitive and it evaluates to False ...
        if (type(value) not in [int, float, bool]) and not bool(value):
            # it's a null pointer
            value = None
        elif hasattr(value, "_length_") and hasattr(value, "_type_"):
            # Probably an array
            value = np.array(list(value))
        elif hasattr(value, "_fields_"):
            # Probably another struct
            value = getdict(value)
        result[field] = value
    return result



from PySide6.QtGui import QColor, QBrush
from PySide6.QtCore import Slot, Qt
from PySide6.QtWidgets import QFrame, QVBoxLayout
from PySide6.QtCharts import QChart, QChartView, QLineSeries, QValueAxis
from ..common_widgets import Label


SAMPLE_COUNT = 10000


class AudioSensor(QFrame):
    def __init__(self) -> None:
        super().__init__()
        self.setMaximumWidth(320)
        self.setMaximumHeight(480)
        self.setObjectName("AudioSensor")

        self.setStyleSheet(
            " QFrame#AudioSensor { border-color: white; }"
        )
        
        self.layout_main = QVBoxLayout()
        self.title = Label("Audio Sensor", orientation=Qt.AlignmentFlag.AlignCenter)
        self.title.setFixedHeight(60)

        self.series = QLineSeries()
        self.chart = QChart()
        self.chart.setTheme(QChart.ChartThemeDark)
        self.chart.addSeries(self.series)
        self.axis_x = QValueAxis()
        self.axis_x.setRange(0, SAMPLE_COUNT)
        self.axis_x.setLabelFormat("%g")
        self.axis_y = QValueAxis()
        self.axis_y.setRange(-1, 1)
        self.chart.setAxisX(self.axis_x, self.series)
        self.chart.setAxisY(self.axis_y, self.series)
        self.chart.legend().hide()
        self.chart_view = QChartView(self.chart)

        self.layout_main.addWidget(self.title)
        self.layout_main.addWidget(self.chart_view)
        self.setLayout(self.layout_main)
